/*
 * Realtek Semiconductor Corp.
 *
 * rlxboard.c:
 *	uboot bsp file
 *
 * Jethro Hsu (jethro@realtek.com)
 */

#include <common.h>
#include <command.h>
#include <asm/mipsregs.h>
#include <asm/io.h>
#include <asm/gpio.h>
#include <netdev.h>
#include "bspchip.h"
#include "ddr_def.h"
#include <dma.h>

#define NAND_MEM_SD			0x300
#define ETH_MEM_SD			0x80
#define CIPHER_MEM_SD			0x40
#define AUDIO_MEM_SD			0x20
#define H264_MEM_SD			0x18
#define U2DEV_MEM_SD			0x04
#define SDIO_MEM_SD			0x03

#define get_val(addr)		REG32(addr)
#define set_val(addr, val)	REG32(addr) = (val)
#define clear_bit(addr, val)	set_val((addr), (get_val(addr) & ~(val)))
#define set_bit(addr, val)	set_val((addr), (get_val(addr) | (val)))

phys_size_t initdram(int board_type)
{
	u32 drr_value_temp;
	u16 ddrc_tref_value;
	u16 pctl_period_ps;
	/* Sdram is setup by assembler code */
	/* If memory could be changed, we should return the true value here */
	/*return CONFIG_SYS_SDRAM_SIZE;*/
	/*
	 *probe DDR size ,
	 *1. use uncachable address to probe size ,
	 *if use cachable address , the value
	 *will save in cache , the addess seemed not wrapped,
	 *2. for 128MB flash, the address 0xAFFFFFFC will wrap to 0x0xA7FFFFFC,
	 *for 64MB flash , the address 0xA7FFFFFC will wrap to 0xA3FFFFFC
	 */

	MEM32(0xAFFFFFFC) = 0x10000000;
	MEM32(0xA7FFFFFC) = 0x08000000;
	MEM32(0xA3FFFFFC) = 0x04000000;
	MEM32(0xA1FFFFFC) = 0x02000000;

	/*When ddr size is 128MB, change trfc value.
	 *Default value is for 256MB
	 *For 256MB, trfc is 160500.
	 *For 128MB, trfc is 110000.
	 *Using tref value to calculate pctl_period_ps.
	 */
	if (MEM32(0xAFFFFFFC) == 0x08000000) {
		drr_value_temp = MEM32(DDRC_BASE + PCTL_DRR);
		ddrc_tref_value = (u16)(drr_value_temp >> 8);
		pctl_period_ps = 60000000/(ddrc_tref_value + 0x100);
		MEM32(DDRC_BASE + PCTL_DRR) = (drr_value_temp & 0xffffff00) |
			(110000/pctl_period_ps + 1);
		MEM32(DDRC_BASE + PCTL_CCR) = 0x80000000;
	}

	return MEM32(0xAFFFFFFC);

}

void dump_work_mode(void)
{

	if ((REG32(RST_LOAD_MODE_REG) & WORK_MODE_SELECT_MASK)
			== MODE_FORCE_NOR)
		printf("force spi nor mode\n");

	else if ((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DOWNLOAD_NOR)
		printf("spi nor rescue mode\n");

	else if ((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DOWNLOAD_NAND)
		printf("spi nand rescue mode\n");

	else if ((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DOWNLOAD_EMMC)
		printf("emmc rescue mode\n");

	else {
		if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
					== MODE_BOOT_NOR) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			 == MODE_JTAG_NOR))
			printf("spi nor boot mode\n");

		if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
					== MODE_BOOT_NAND)  ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			 == MODE_JTAG_NAND))
			printf("spi nand boot mode\n");

		if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
					== MODE_BOOT_EMMC)  ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			 == MODE_JTAG_EMMC))
			printf("emmc boot mode\n");
	}
}

int checkboard(void)
{
	u32 prid;
	u32 cpufreq;

	prid = read_c0_prid();
	prid &= 0xffff;
	cpufreq = CPU_FREQUCNCY;

	REG32(CLK_CHANGE) = CPU_CLK_CHANGE;
#if (defined CONFIG_CPU_500M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL1_2;
#elif (defined CONFIG_CPU_250M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL1_2|CLOCK_SELECT_DIV2;
#elif (defined CONFIG_CPU_200M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL0_3|CLOCK_SELECT_DIV2;
#elif (defined CONFIG_CPU_600M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL0_2;
#elif (defined CONFIG_CPU_400M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE | (0x0D << 16) | CPU_SELECT_PLL1_2;
#elif (defined CONFIG_CPU_300M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE | (0x1A << 16) | CPU_SELECT_PLL1_2;
#endif
	REG32(CLK_CHANGE) = 0;

#ifdef CONFIG_BOARD_RTS3903
#ifdef CONFIG_RTS3903_QFN88_DDR2_256MBIT
	printf("Board: IPCAM RTS3905 CPU: %dM :", cpufreq / 1000000);
#else
#if (defined CONFIG_RTS3903_QFN88_DDR2_512MBIT) ||\
	(defined CONFIG_RTS3903_BGA151_DDR2_512MBIT)
	printf("Board: IPCAM RTS3903 CPU: %dM :", cpufreq / 1000000);
#else
	printf("Board: IPCAM RTS3904 CPU: %dM :", cpufreq / 1000000);
#endif
#endif
#endif
	switch (prid) {
	case 0x0000dc06:
		printf("rx5381");
		break;
	case 0x0000dc05:
		printf("rx4381");
		break;
	case 0x0000dc04:
		printf("rx5271");
		break;
	case 0x0000dc03:
		printf("rx4271");
		break;
	case 0x0000dc02:
		printf("rx5281");
		break;
	case 0x0000dc01:
		printf("rx4281");
		break;
	default:
		printf("unknown");
	}
	printf(" prid=0x%x\n", prid);

	dump_work_mode();
	return 0;
}

u8 get_rst_mode(void)
{
	u8 board_reset_mode;

	if ((REG32(RST_LOAD_MODE_REG) & WORK_MODE_SELECT_MASK)
			== MODE_FORCE_NOR)
		board_reset_mode = NOR_MODE;

	else if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_BOOT_NOR) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			 == MODE_JTAG_NOR))
		board_reset_mode = NOR_MODE;

	else if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_BOOT_NAND) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_JTAG_NAND))
		board_reset_mode = NAND_MODE;

	else if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_BOOT_EMMC)  ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DEBUG_BOOT_EMMC) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_JTAG_EMMC))
		board_reset_mode = EMMC_MODE;

	else {
		if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DOWNLOAD_NOR) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DEBUG_DOWNLOAD_NOR))
		board_reset_mode = NOR_RES_MODE;

		if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DOWNLOAD_NAND) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DEBUG_DOWNLOAD_NAND))
		board_reset_mode = NAND_RES_MODE;

		if (((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DOWNLOAD_EMMC) ||
			((REG32(RST_LOAD_MODE_REG) & RST_MODE_SELECT_MASK)
			== MODE_DEBUG_DOWNLOAD_EMMC))
		board_reset_mode = EMMC_RES_MODE;
	}
	/*printf("rst mode is %x\n", board_reset_mode);*/

	return board_reset_mode;
}

void rts_sys_mem_up(int model)
{
	switch (model) {
	case SYS_MEM_SD_NAND_SPIC:
		clear_bit(SYS_MEM_SD, NAND_MEM_SD);
		break;

	case SYS_MEM_SD_ETH:
		clear_bit(SYS_MEM_SD, ETH_MEM_SD);
		break;

	case SYS_MEM_SD_CIPHER:
		clear_bit(SYS_MEM_SD, CIPHER_MEM_SD);
		break;

	case SYS_MEM_SD_AUDIO:
		clear_bit(SYS_MEM_SD, AUDIO_MEM_SD);
		break;

	case SYS_MEM_SD_H264:
		clear_bit(SYS_MEM_SD, H264_MEM_SD);
		break;

	case SYS_MEM_SD_U2DEV:
		clear_bit(SYS_MEM_SD, U2DEV_MEM_SD);
		break;

	case SYS_MEM_SD_SDIO:
		clear_bit(SYS_MEM_SD, SDIO_MEM_SD);
		break;

	default:
		printf("ERROR: invalid sys mem model %d\n", model);
		break;
	}
}

void reset_eth_phy(void)
{
	u32 reg;

	reg = get_val(0xb8861024);
	set_val(0xb8861024, (reg & 0xfffbbfff));
}

void rts_sys_mem_down(int model)
{
	switch (model) {
	case SYS_MEM_SD_NAND_SPIC:
		set_bit(SYS_MEM_SD, NAND_MEM_SD);
		break;

	case SYS_MEM_SD_ETH:
		set_bit(SYS_MEM_SD, ETH_MEM_SD);
		break;

	case SYS_MEM_SD_CIPHER:
		set_bit(SYS_MEM_SD, CIPHER_MEM_SD);
		break;

	case SYS_MEM_SD_AUDIO:
		set_bit(SYS_MEM_SD, AUDIO_MEM_SD);
		break;

	case SYS_MEM_SD_H264:
		set_bit(SYS_MEM_SD, H264_MEM_SD);
		break;

	case SYS_MEM_SD_U2DEV:
		set_bit(SYS_MEM_SD, U2DEV_MEM_SD);
		break;

	case SYS_MEM_SD_SDIO:
		set_bit(SYS_MEM_SD, SDIO_MEM_SD);
		break;

	default:
		printf("ERROR: invalid sys mem model %d\n", model);
		break;
	}
}

static u64 integer_div(u64 dividend, u64 divisor)
{
	u64 res = 0;

	if (divisor == 0) {
		printf("wrong divisor\n");
		return 0;
	}

	if (dividend < divisor)
		return 0;

	while (dividend > divisor) {
		dividend -= divisor;
		res++;
	}

	if (dividend == divisor)
		res++;

	return res;
}

int get_dram_clock(void)
{
#ifdef CONFIG_TARGET_ASIC
	u64 dpi_n_code, dpi_f_code;

	dpi_f_code = MEM32(DDR_PHY_SSC2) & 0x7ff;
	dpi_n_code = MEM32(DDR_PHY_SSC3) & 0xff;

	u64 clock = integer_div(REF_PLL * dpi_f_code * 2, 2048);

	clock = integer_div((clock + REF_PLL * (dpi_n_code + 2) * 2 + 500000),
			    1000000);
	return (int)clock;
#endif
#ifdef CONFIG_TARGET_FPGA
	return 224;
#endif
	return 0;
}

void _machine_restart(void)
{
	/* set watchdog */
	/* watchdog time: 1s, reset system when timeout, enable watchdog */
	writel(WATCH_DOG_CFG_DATA, (u32 *)WATCH_DOG_CFG_REG);
	printf("set watchdog, resetting...");
	while (1)
		;
}

#if defined(CONFIG_DISPLAY_CPUINFO)
/*
 * Print CPU information
 */
int print_cpuinfo(void)
{
	puts("CPU  : SHEIPA SOC");

	return 0;
}
#endif

#if defined(CONFIG_DM_SPI_FLASH) && !defined(CONFIG_SYS_NO_FLASH)
int init_dm_spi_flash_sheipa(void)
{
	struct udevice *dev;
	const char flash_name[] = "flash@0";

	uclass_first_device(UCLASS_SPI_FLASH, &dev);
	if (!dev) {
		debug("sheipa flash init failed");
		return 1;
	}
	while (strcmp(dev->name, flash_name) != 0) {
		uclass_next_device(&dev);
		if (!dev) {
			debug("sheipa flash init failed");
			return 1;
		}
	}
	return 0;
}
#endif

#if defined(CONFIG_BOARD_EARLY_INIT_R)
/*
 * init spi flash
 */
int board_early_init_r(void)
{
#if defined(CONFIG_DM_SPI_FLASH) && !defined(CONFIG_NO_SYS_FLASH)
	return init_dm_spi_flash_sheipa();
#endif
	return 0;
}

#endif
