/*
 * Realtek Semiconductor Corp.
 *
 * rlxboard.c:
 *	uboot bsp file
 *
 * Jethro Hsu (jethro@realtek.com)
 */

#include <common.h>
#include <command.h>
#include <asm/mipsregs.h>
#include <asm/io.h>
#include <asm/gpio.h>
#include <netdev.h>
#include "bspchip.h"
#include "ddr_def.h"
#include <dma.h>

#define NAND_MEM_SD			0x300
#define ETH_MEM_SD			0x80
#define CIPHER_MEM_SD			0x40
#define AUDIO_MEM_SD			0x20
#define H264_MEM_SD			0x18
#define U2DEV_MEM_SD			0x04
#define SDIO_MEM_SD			0x03

#define get_val(addr)		REG32(addr)
#define set_val(addr, val)	REG32(addr) = (val)
#define clear_bit(addr, val)	set_val((addr), (get_val(addr) & ~(val)))
#define set_bit(addr, val)	set_val((addr), (get_val(addr) | (val)))

/*#define _DDR_SETTING_DEBUG_*/
phys_size_t initdram(int board_type)
{
#ifdef _DDR_SETTING_DEBUG_
	printf("DDR controller setting\n");
	printf("2a0 wrap_idr: %x\n", MEM32(DDRC_BASE + PCTL_WRAP_IDR));
	printf("f4 svn_id: %x\n", MEM32(DDRC_BASE + PCTL_SVN_ID));
	printf("f8 idr: %x\n", MEM32(DDRC_BASE + PCTL_IDR));
	printf("224 misc: %x\n", MEM32(DDRC_BASE + PCTL_MISC));
	printf("4 dcr: %x\n", MEM32(DDRC_BASE + PCTL_DCR));
	printf("08 iocr: %x\n", MEM32(DDRC_BASE + PCTL_IOCR));
	printf("3c mr2: %x\n", MEM32(DDRC_BASE + PCTL_MR2));
	printf("38 mr1: %x\n", MEM32(DDRC_BASE + PCTL_MR1));
	printf("34 mr0: %x\n", MEM32(DDRC_BASE + PCTL_MR0));
	printf("10 drr: %x\n", MEM32(DDRC_BASE + PCTL_DRR));
	printf("14 tpr0: %x\n", MEM32(DDRC_BASE + PCTL_TPR0));
	printf("18 tpr1: %x\n", MEM32(DDRC_BASE + PCTL_TPR1));
	printf("1c tpr2: %x\n", MEM32(DDRC_BASE + PCTL_TPR2));
	printf("0c csr: %x\n", MEM32(DDRC_BASE + PCTL_CSR));
	printf("30 mrinfo: %x\n", MEM32(DDRC_BASE + PCTL_MR_INFO));
#endif
	/* Sdram is setup by assembler code */
	/* If memory could be changed, we should return the true value here */
	/*
	 *probe DDR size ,
	 *1. use physical address to probe size ,
	 *if use cachable address , the value
	 *will save in cache , the addess seemed not wrapped,
	 *2. the ddr physical address is from 0x80000000~0x9fffffff,
	 *we can use memory wrap method to detect size
	 */

	MEM32(0x98910000) = 0x20000000;
	dma_copy(0x18910000, 0x9FFFFFFC, 4);

	MEM32(0x98910000) = 0x10000000;
	dma_copy(0x18910000, 0x8FFFFFFC, 4);

	MEM32(0x98910000) = 0x8000000;
	dma_copy(0x18910000, 0x87FFFFFC, 4);

	MEM32(0x98910000) = 0x4000000;
	dma_copy(0x18910000, 0x83FFFFFC, 4);

	dma_copy(0x9FFFFFFC, 0x18910000, 4);
	printf(" size is 0x%x\n", MEM32(0x98910000));
	return MEM32(0x98910000);

}

void dump_work_mode(void)
{
	if ((REG32(RST_LOAD_MODE_REG) &
			WORK1_MODE_SEL_MASK)
			== NOR)
		printf("spi nor boot mode\n");
	else {
		if ((REG32(RST_LOAD_MODE_REG) &
			WORK2_MODE_SEL_MASK)
			== NOR_JTAG)
			printf("spi nor boot mode\n");
		else
			printf("spi nor rescue mode\n");
	}
}

int checkboard(void)
{
	u32 prid;
	u32 cpufreq;
	prid = read_c0_prid();
	prid &= 0xffff;
	cpufreq = CPU_FREQUCNCY;

	/*CPU 500M  PLL0/2 div1
	 *ew 0xb8861000 0x20
	 *ew 0xb8861010 0x104
	 *ew 0xb8861000 0x00
	 */
	REG32(CLK_CHANGE) = CPU_CLK_CHANGE;
#if (defined CONFIG_CPU_500M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL1_2;
#elif (defined CONFIG_CPU_250M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL1_2|CLOCK_SELECT_DIV2;
#elif (defined CONFIG_CPU_200M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL0_3|CLOCK_SELECT_DIV2;
#elif (defined CONFIG_CPU_600M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE|CPU_SELECT_PLL0_2;
#elif (defined CONFIG_CPU_400M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE | (0x0D << 16) | CPU_SELECT_PLL1_2;
#elif (defined CONFIG_CPU_300M)
	REG32(CPU_CLK_CFG_REG) = CLK_ENABLE | (0x1A << 16) | CPU_SELECT_PLL1_2;
#endif
	REG32(CLK_CHANGE) = 0;

	printf("Board: IPCAM RTS3913 CPU: %dM :", cpufreq / 1000000);

	switch (prid) {
	case 0x0000dc06:
		printf("rx5381");
		break;
	case 0x0000dc05:
		printf("rx4381");
		break;
	case 0x0000dc04:
		printf("rx5271");
		break;
	case 0x0000dc03:
		printf("rx4271");
		break;
	case 0x0000dc02:
		printf("rx5281");
		break;
	case 0x0000dc01:
		printf("rx4281");
		break;
	default:
		printf("unknown");
	}
	printf(" prid=0x%x\n", prid);

	dump_work_mode();
	return 0;
}

u8 get_rst_mode(void)
{
	u8 board_reset_mode = 0;

	if ((REG32(RST_LOAD_MODE_REG) & WORK1_MODE_SEL_MASK)
			== NOR)
		board_reset_mode = NOR_MODE;
	else {
		if ((REG32(RST_LOAD_MODE_REG) &
			WORK2_MODE_SEL_MASK)
			== NOR_JTAG)
			board_reset_mode = NOR_MODE;
		else
			board_reset_mode = NOR_RES_MODE;
	}

	return board_reset_mode;
}

void rts_sys_mem_up(int model)
{
	switch (model) {
	case SYS_MEM_SD_NAND_SPIC:
		clear_bit(SYS_MEM_SD, NAND_MEM_SD);
		break;

	case SYS_MEM_SD_ETH:
		clear_bit(SYS_MEM_SD, ETH_MEM_SD);
		break;

	case SYS_MEM_SD_CIPHER:
		clear_bit(SYS_MEM_SD, CIPHER_MEM_SD);
		break;

	case SYS_MEM_SD_AUDIO:
		clear_bit(SYS_MEM_SD, AUDIO_MEM_SD);
		break;

	case SYS_MEM_SD_H264:
		clear_bit(SYS_MEM_SD, H264_MEM_SD);
		break;

	case SYS_MEM_SD_U2DEV:
		clear_bit(SYS_MEM_SD, U2DEV_MEM_SD);
		break;

	case SYS_MEM_SD_SDIO:
		clear_bit(SYS_MEM_SD, SDIO_MEM_SD);
		break;

	default:
		printf("ERROR: invalid sys mem model %d\n", model);
		break;
	}
}

void reset_eth_phy(void)
{
	u32 reg;

	reg = get_val(0xb8861024);
	set_val(0xb8861024, (reg & 0xfffbbfff));
}

void rts_sys_mem_down(int model)
{
	switch (model) {
	case SYS_MEM_SD_NAND_SPIC:
		set_bit(SYS_MEM_SD, NAND_MEM_SD);
		break;

	case SYS_MEM_SD_ETH:
		set_bit(SYS_MEM_SD, ETH_MEM_SD);
		break;

	case SYS_MEM_SD_CIPHER:
		set_bit(SYS_MEM_SD, CIPHER_MEM_SD);
		break;

	case SYS_MEM_SD_AUDIO:
		set_bit(SYS_MEM_SD, AUDIO_MEM_SD);
		break;

	case SYS_MEM_SD_H264:
		set_bit(SYS_MEM_SD, H264_MEM_SD);
		break;

	case SYS_MEM_SD_U2DEV:
		set_bit(SYS_MEM_SD, U2DEV_MEM_SD);
		break;

	case SYS_MEM_SD_SDIO:
		set_bit(SYS_MEM_SD, SDIO_MEM_SD);
		break;

	default:
		printf("ERROR: invalid sys mem model %d\n", model);
		break;
	}
}

void _machine_restart(void)
{
	/* set watchdog */
	/* watchdog time: 1s, reset system when timeout, enable watchdog */
	writel(WATCH_DOG_CFG_DATA, (u32 *)WATCH_DOG_CFG_REG);
	printf("set watchdog, resetting...");
	while (1)
		;
}
#ifdef CONFIG_TARGET_ASIC
static s64 integer_div(s64 dividend, s64 divisor)
{
	s64 res = 0;

	if (divisor == 0) {
		printf("wrong divisor\n");
		return 0;
	}

	if (dividend < divisor)
		return 0;

	while (dividend > divisor) {
		dividend -= divisor;
		res++;
	}

	if (dividend == divisor)
		res++;

	return res;
}
#endif
int get_dram_clock(void)
{
#ifdef CONFIG_TARGET_ASIC
	u64 dpi_n_code, dpi_f_code;

	dpi_f_code = MEM32(DDR_PHY_SSC2) & 0x7ff;
	dpi_n_code = MEM32(DDR_PHY_SSC3) & 0xff;
	s64 clock = integer_div(REF_PLL * dpi_f_code * 2, 2048);

	clock = integer_div((clock + REF_PLL * (dpi_n_code + 2) * 2 + 500000),
			    1000000);
	return (int)clock;
#endif
#ifdef CONFIG_TARGET_FPGA
	return 224;
#endif
	return 0;
}

#if defined(CONFIG_BOARD_EARLY_INIT_R)
/*
 * init spi flash
 */
int board_early_init_r(void)
{
#if defined(CONFIG_DM_SPI_FLASH) && !defined(CONFIG_NO_SYS_FLASH)
	return init_dm_spi_flash_sheipa();
#endif
	return 0;
}

#endif
