/*
 * Realtek Semiconductor Corp.
 *
 * bsp.h
 * Board Support Package header file
 *
 * Tony Wu (tonywu@realtek.com.tw)
 * Dec. 07, 2007
 */

#ifndef _BSPCHIP_H_
#define _BSPCHIP_H_

/*
 ******************************
 * DEFINITIONS AND TYPES
 ******************************
 */

/* Register Macro */
#ifndef REG64
#define REG64(reg)      (*(volatile u64 *)(reg))
#endif
#ifndef REG32
#define REG32(reg)      (*(volatile u32 *)(reg))
#endif
#ifndef REG16
#define REG16(reg)      (*(volatile u16 *)(reg))
#endif
#ifndef REG8
#define REG8(reg)       (*(volatile u8  *)(reg))
#endif

/*Memory Macro*/
#ifndef MEM32
#define MEM32(addr)	(*(volatile u32 *)(addr))
#endif

/*
 * Arbiter controller
 */
#define BSP_ARBITER_VADDR	0xB80F0400
#define BSP_ARBITER_REG(x) ((x) + BSP_ARBITER_VADDR)
#define BSP_ARBITER_CTRL0	BSP_ARBITER_REG(0)
#define BSP_ARBITER_CTRL1	BSP_ARBITER_REG(0x04)
#define BSP_ARBITER_CTRL2	BSP_ARBITER_REG(0x08)
#define BSP_ARBITER_CTRL3	BSP_ARBITER_REG(0x0c)

/*
 ******************************
 * Timer
 ******************************
 */
/* Timer0 register base address mapping */
#define TIMER0_IOBASE		0xB8810000
/*The frequency of timer module*/
#define TIMER0_FREQ		25000000

/* Register offset from TIMERx_IOBASE */
#define TIMER_TLCR1		0x00
#define TIMER_TCV		0x04
#define TIMER_TCR		0x08
#define TIMER_EOIA		0x0c

/*
 ******************************
 * Uart
 ******************************
 */
/* Uart1 register base address mapping */
#define UART1_IOBASE		0xB8810100
/* Serial Clock	*/
#define UART1_FREQ			24000000
#define XB2_UART1_PULL_UP_CTRL 0xb887003c
#define XB2_UART1_PULL_DOWN_CTRL 0xb8870030

#define XB2_UART2_PULL_CTRL 0xb8870048
#define NO_PULL 0
#define PULL_DOWN 1
#define PULL_UP 2
#define UART_RX_PULL_UP (PULL_UP << 2)
#define UART_RX_PULL_DOWN (PULL_DOWN << 2)

/* Register offset from UARTx_IOBASE */
#define UART_RBR		0x00
#define UART_THR		0x00
#define UART_IER		0x04
#define UART_IIR		0x08
#define UART_LCR		0x0C
#define UART_LSR		0x14
#define UART_DLL		0x00
#define UART_DLH		0x04
#define UART_CTR		0xFC

/* uart LCR register bits */
#define UART_LCR_PEN		0x00000008	/* parity enable bit */
#define UART_LCR_EPS		0X00000010
#define UART_LCR_PARITY		(UART_LCR_PEN | UART_LCR_EPS)
#define UART_LCR_STOP		0x00000004	/* stop bit          */
#define UART_LCR_DLS		0x00000003	/* 8 bits mode       */
#define UART_LCR_DLAB		0x00000080	/* DLAB bit          */

#define UART_DL_BAUD_9600		(UART_BAUD_BASE / 9600)
#define UART_DL_BAUD_19200		(UART_BAUD_BASE / 19200)
#define UART_DL_BAUD_38400		(UART_BAUD_BASE / 38400)
#define UART_DL_BAUD_57600		(UART_BAUD_BASE / 57600)
#define UART_DL_BAUD_115200		(UART_BAUD_BASE / 115200)

/* baudrate platform specific setting */
#define UART_BAUD_BASE			(UART1_FREQ >> 4)
#ifdef CONFIG_BAUDRATE_57600
#define UART_BAUD_DEFAULT		UART_DL_BAUD_57600
#endif
#ifdef CONFIG_BAUDRATE_115200
#define UART_BAUD_DEFAULT		UART_DL_BAUD_115200
#endif

#define SYS_CLK_BASE 0xb88d0000
#define SYSCLK_REG(x) ((x) + SYS_CLK_BASE)

#define CLK_CHANGE		SYSCLK_REG(0x00)
#define BUS_CLK_CHANGE			0x80
#define XB2_CLK_CHANGE			0x40
#define CPU_CLK_CHANGE			0x20
#define DRAM_CLK_CHANGE		0x10
#define CLK_ENABLE 0x1000000

#define CLOCK_SELECT_DIV2	(0x1 << 2)
#define CLOCK_SELECT_DIV4	(0x2 << 2)
#define CLOCK_SELECT_DIV6	(0x3 << 2)
#define CLOCK_SELECT_DIV8	(0x4 << 2)
#define CLOCK_SELECT_DIV10	(0x5 << 2)
#define CLOCK_SELECT_DIV12	(0x6 << 2)
#define CLOCK_SELECT_DIV14	(0x7 << 2)

#define DRAM_OCP_BUS_CLK_CFG_REG		SYSCLK_REG(0x0C)
#define DRAMC_SELECT_USBPLL_2	0
#define DRAMC_SELECT_GPLL0_2	0x01
#define DRAMC_SELECT_USBPLL_1	0x02
#define DRAMC_SELECT_GPLL1_2	0x03

#define CPU_CLK_CFG_REG		SYSCLK_REG(0x10)
#define CPU_SELECT_USBPLL_3	0
#define CPU_SELECT_GPLL0_1		0x01
#define CPU_SELECT_GPLL1_1		0x02
#define CPU_SELECT_USBPLL_1	0x03

#define XB2_CLK_CFG_REG		SYSCLK_REG(0x14)
#define XB2_SELECT_USBPLL_2	0
#define XB2_SELECT_GPLL0_5	0x01
#define XB2_SELECT_GPLL0_2	0x02

#define BUS_CLK_CFG_REG		SYSCLK_REG(0x18)
#define BUS_SELECT_USBPLL_2	0
#define BUS_SELECT_GPLL0_2	0x01
#define BUS_SELECT_GPLL0_5	0x02
#define BUS_SELECT_GPLL1_7	0x03

#define ETHERNET_CLK_CFG_REG		SYSCLK_REG(0xc4)

#define MAC_BYPASS_CLK_CFG_REG	SYSCLK_REG(0xcc)
#define MAC_BYPASS_CLK_EN			0x01000000
#define MIPI_BYPASS_CLK_GPLL0_DIV2	0

#define UART_CLK_CFG_REG		SYSCLK_REG(0x28)
#define UART_SELECT_USBPLL_5    0x0
#define UART_SELECT_GPLL0_5    0x01
#define UART_SELECT_GPLL1_5    0x02
#define EFUSE_CLK_CFG_REG	SYSCLK_REG(0xc8)
#define DMA_CLK_CFG_REG		SYSCLK_REG(0xd8)

#define SYS_GPLL_BASE	0xb88b0000
#define SYSGPLL_REG(x) ((x) + SYS_GPLL_BASE)
#define SYS_GPLL0_CTRL			SYSGPLL_REG(0x00)
#define SYS_GPLL0_CFG			SYSGPLL_REG(0x04)
#define SYS_GPLL0_SCCG_CFG0	SYSGPLL_REG(0x08)
#define SYS_GPLL0_SCCG_CFG1	SYSGPLL_REG(0x0c)
#define SYS_GPLL0_STATUS		SYSGPLL_REG(0x10)
#define SYS_GPLL0_SCCG_CFG2	SYSGPLL_REG(0x14)
#define SYS_GPLL0_SCCG_CFG3	SYSGPLL_REG(0x18)

#define SYS_GPLL1_CTRL			SYSGPLL_REG(0x100)
#define SYS_GPLL1_CFG			SYSGPLL_REG(0x104)
#define SYS_GPLL1_SCCG_CFG0	SYSGPLL_REG(0x108)
#define SYS_GPLL1_SCCG_CFG1	SYSGPLL_REG(0x10c)
#define SYS_GPLL1_STATUS		SYSGPLL_REG(0x110)
#define SYS_GPLL1_SCCG_CFG2	SYSGPLL_REG(0x114)
#define SYS_GPLL1_SCCG_CFG3	SYSGPLL_REG(0x118)

#define SYS_GPLL2_CTRL			SYSGPLL_REG(0x200)
#define SYS_GPLL2_CFG			SYSGPLL_REG(0x204)
#define SYS_GPLL2_SCCG_CFG0	SYSGPLL_REG(0x208)
#define SYS_GPLL2_SCCG_CFG1	SYSGPLL_REG(0x20c)
#define SYS_GPLL2_STATUS		SYSGPLL_REG(0x210)
#define SYS_GPLL2_SCCG_CFG2	SYSGPLL_REG(0x214)
#define SYS_GPLL2_SCCG_CFG3	SYSGPLL_REG(0x218)

#define SYS_GPLL3_CTRL			SYSGPLL_REG(0x200)
#define SYS_GPLL3_CFG			SYSGPLL_REG(0x204)
#define SYS_GPLL3_SCCG_CFG0	SYSGPLL_REG(0x208)
#define SYS_GPLL3_SCCG_CFG1	SYSGPLL_REG(0x20c)
#define SYS_GPLL3_STATUS		SYSGPLL_REG(0x210)
#define SYS_GPLL3_SCCG_CFG2	SYSGPLL_REG(0x214)
#define SYS_GPLL3_SCCG_CFG3	SYSGPLL_REG(0x218)

#define SYS_GPLL_STABLE			0x01

#define PLL_EN_WD_PLL		0x01
#define CMU_EN_CKOOBS_PLL 0x02
#define LDO_EN_PLL			0x04
#define CMU_EN_PLL			0x08
#define PLL_SETP1			0x02
#define PLL_SETP2			0x03

#define	SPIC_VA	0xB8030000
#define	SPICBASE_REG(x)	((x) + SPIC_VA)
#define	SPIC_BAUD_REG	SPICBASE_REG(0x14)

#define SPIC_BAUDR	0x14
#define SPIC_READ_DUAL_DATA	0xE4
#define SPIC_AUTO_LENGTH	0x11C
#define	SPIC_VALID_CMD	0x120

#define SNFC_BAUDR	0xb8038008
#define SNFC_AUTO_LENGTH	0xb8038024

/* some register about PWM and GPIO */
#define GPIO_BASE_ADDR	0xb8800000
#define XB2_SPI_PULLCTRL	(GPIO_BASE_ADDR + 0x28)
#define ALL_SPI_NOPULL	0
#define GPIO_ETN_LED_SEL_REG	(GPIO_BASE_ADDR + 0x006c)
#define GPIO_ETN_LED_SEL_PWM	0
#define SD0_PULLCTRL	(GPIO_BASE_ADDR + 0x00c8)
#define SD0_PULLCTRL_SETTING	0xa6aa

#define PWM_BASE_ADDR		0xb8820040
#define PWM_EN_REG		(PWM_BASE_ADDR + 0)
#define PWM_KEEP_REG		(PWM_BASE_ADDR +	0x04)
#define PWM_HIGH_REG		(PWM_BASE_ADDR + 0x08)
#define PWM_LOW_REG		(PWM_BASE_ADDR + 0x0c)
#define PWM_NUM_REG		(PWM_BASE_ADDR + 0x14)
#define PWM_INV_REG		(PWM_BASE_ADDR + 0x18)

/*ethernet config*/
#define ETHERNET_CFG_REG	0xb8861024
#define ETH_EPHY_ADDR	(1 << 9)
#define AFE_POW_STATE	(1 << 18)
#define ETH_EPHY_CLK_BYPASS	(1 << 27)

#define FEPHY_LDO_CTRL		0xb886102c

#define LDO_FEPHY_CFG		0xb8861030

#define ETH_PDWN_CONTROL	0xb88dd008

/* some register about reset */
#define SYS_RESET_BASE	0xb88d8000
#define SYSRESET_REG(x) ((x) + SYS_RESET_BASE)

#define FORCE_REG_RST_FWC	SYSRESET_REG(0x4)
#define FORCE_FEPHY_RST	(1 << 5)

#define FORCE_REG_ASYNC_RST	SYSRESET_REG(0x8)
#define FORCE_ETHERNET_RST	(1 << 6)
#define FORCE_EFUSE_RST	BIT(17)

#define WATCHDOG_CFG_REG SYSRESET_REG(0x2000)
#define WATCHDOG_CTL	SYSRESET_REG(0x2004)
#define WATCHDOG_INT_EN	SYSRESET_REG(0x2008)
#define WATCHDOG_INT_FLAG	SYSRESET_REG(0x200c)

#define WATCH_DOG_PAD_PULLDOWN		0x1
#define WATCH_DOG_TIMEOUT_1S		0x0
#define WATCH_DOG_TIMEOUT_2S		0x1
#define WATCH_DOG_TIMEOUT_4S		0x2
#define WATCH_DOG_TIMEOUT_8S		0x3
#define WATCH_DOG_TIMEOUT_16S		0x4
#define WATCH_DOG_TIMEOUT_32S		0x5
#define WATCH_DOG_TIMEOUT_64S		0x6
#define WATCH_DOG_TIMEOUT_MASK		(0xF << 16)

#define WATCH_DOG_RESET_SYSTEM		0x1
#define WATCH_DOG_ENABLE		0x1
#define WATCH_DOG_CFG_DATA		((WATCH_DOG_PAD_PULLDOWN << 9) \
					| (WATCH_DOG_TIMEOUT_1S << 16) \
					| (WATCH_DOG_RESET_SYSTEM << 1) \
					| (WATCH_DOG_ENABLE << 0))

/*set UART2 RX to GPIO for WP#*/
#define GPIO_UART2_RXD_BIT	5
#define GPIO_SELECT	1
#define GPIO_OUTPUT_ENABLE	1
#define GPIO_OUTPUT_VALUE	1

/*Share pin config*/
#define	XB2_GPIO_BASE	0xb8800000
#define GPIO_OE				(XB2_GPIO_BASE + 0x0000)
#define UART0_GPIO_OE			(XB2_GPIO_BASE + 0x0020)
#define UART1_GPIO_OE			(XB2_GPIO_BASE + 0x0040)
#define UART2_GPIO_OE			(XB2_GPIO_BASE + 0x0060)
#define PWM_GPIO_OE			(XB2_GPIO_BASE + 0x0080)
#define XB2_I2C_GPIO_OE			(XB2_GPIO_BASE + 0x00a0)
#define SD0_GPIO_OE			(XB2_GPIO_BASE + 0x00c0)
#define SD1_GPIO_OE			(XB2_GPIO_BASE + 0x00e0)
#define VIDEO_GPIO_OE			(XB2_GPIO_BASE + 0x0100)
#define DMIC_GPIO_OE			(XB2_GPIO_BASE + 0x0130)
#define AUDIO_ADDA_GPIO_OE		(XB2_GPIO_BASE + 0x0150)
#define I2S_GPIO_OE			(XB2_GPIO_BASE + 0x0170)
#define SARADC_GPIO_OE			(XB2_GPIO_BASE + 0x0190)
#define MIPI_TX_GPIO_OE			(XB2_GPIO_BASE + 0x01b0)
#define LVDS_GPIO_OE			(XB2_GPIO_BASE + 0x01d0)
#define USB0_GPIO_OE			(XB2_GPIO_BASE + 0x01f0)
#define USB1_GPIO_OE			(XB2_GPIO_BASE + 0x0210)
#define USB2_GPIO_OE			(XB2_GPIO_BASE + 0x0230)
#define SPI_OE				(XB2_GPIO_BASE + 0x0250)
#define SSOR_I2C_OE			(XB2_GPIO_BASE + 0x0270)

#define XB2_GPIO_REG(x)                 ((x) + XB2_GPIO_BASE)
#define UART1_PULLCTRL XB2_GPIO_REG(0x48)
#define UART1_TX_PE_SHIFT	2
#define UART1_TX_PE_MASK	0xc
#define UART1_RX_PE_SHIFT	0
#define UART1_RX_PE_MASK	3
#define UART1_PE_PU	0x2
#define UART1_PE_PD	0x1

#define UART1_PAD_CFG  XB2_GPIO_REG(0x5c)
#define PAD_XB2_UTXD_SHIFT	4
#define PAD_XB2_UTXD_MASK	0x30
#define UART1_TX_EN	0x2
#define PAD_XB2_URXD_SHIFT	0
#define PAD_XB2_URXD_MASK	3
#define UART1_RX_EN	0x2

#define MIPI_TX_PAD_CFG	XB2_GPIO_REG(0x01cc)
#define ETN_RMII_CFG		0x44444

#define SYS_EFUSE_BASE	0xb88a0000
#define SYSEFUSE_REG(x) ((x) + SYS_EFUSE_BASE)

#define SF_CTRL	SYSEFUSE_REG(0x300)
#define RG_STROBE_SF	BIT(8)
#define RG_MR_SF	BIT(7)
#define RG_RSB_SF	BIT(6)
#define RG_RWL_SF	BIT(5)
#define RG_LOAD_SF	BIT(4)
#define RG_PGENB_SF	BIT(3)
#define RG_CSB_SF	BIT(2)
#define RG_PS_SF	BIT(1)
#define RG_PD_SF	BIT(0)

#define SF_HW_CTRL	SYSEFUSE_REG(0x304)
#define HW_ENABLE	0
#define SW_ENABLE	1

#define	BURST_CTRL	SYSEFUSE_REG(0x308)
#define BURST_LENGTH	0x20

#define HW_CTRL	SYSEFUSE_REG(0x30c)
#define GROUP6_ECC	BIT(12)
#define GROUP5_ECC	BIT(11)
#define GROUP4_ECC	BIT(10)
#define GROUP3_ECC	BIT(9)
#define GROUP2_ECC	BIT(8)
#define GROUP1_ECC	BIT(7)
#define GROUP0_ECC	BIT(6)
#define ASYN_ECC	BIT(5)
#define AUTO_ECC	BIT(4)
#define DOUBLE_BIT	BIT(3)
#define MODE_SELECT_SHIFT 0
#define MODE_SELECT_MASK	7
#define A_READ	0x4
#define A_PGM	0x2
#define R_READ	0x6
#define R_PGM	0x3
#define DOUBLE_BIT_ECC	(DOUBLE_BIT | AUTO_ECC | GROUP6_ECC \
			| GROUP5_ECC | GROUP4_ECC | GROUP3_ECC \
			| GROUP2_ECC | GROUP1_ECC | GROUP0_ECC)

#define HW_START_REG	SYSEFUSE_REG(0x310)
#define HW_START	1

#define ADDR_CTRL	SYSEFUSE_REG(0x318)
#define ADDR_START 0x3e0

#define READ_DELAY_COUNTER	SYSEFUSE_REG(0x31c)
#define	STROBE_READ_DELAY_SHIFT	0
#define	STROBE_READ_DELAY_MASK	0x1f
#define STROBE_READ_VALUE	0x8
#define	PD_SETUP_DELAY_SHIFT	5
#define	PD_SETUP_DELAY_MASK	0xfe0
#define PD_SETUP_VALUE	0x30

#define PROGRAM_DELAY_COUNTER	SYSEFUSE_REG(0x320)

#define	STROBE_WRITE_DELAY_SHIFT	0
#define	STROBE_WRITE_DELAY_MASK	0x7ff
#define STROBE_WRITE_VALUE	0x2cf

#define	PS_TO_CSB_HOLD_DELAY_SHIFT	11
#define	PS_TO_CSB_HOLD_DELAY_MASK	0x7800
#define PS_TO_CSB_HOLD_WRITE_VALUE	0x4

#define	PS_TO_CSB_SETUP_DELAY_SHIFT	15
#define	PS_TO_CSB_SETUP_DELAY_MASK	0x78000
#define PS_TO_CSB_SETUP_WRITE_VALUE	0x4

#define	PD_TO_PS_SETUP_DELAY_SHIFT	19
#define	PD_TO_PS_SETUP_DELAY_MASK	0x3f80000
#define PD_TO_PS_SETUP_WRITE_VALUE	0x30

#define CLEAR_1024_REG	SYSEFUSE_REG(0x328)
#define CLEAR	1

#define EFUSE_VQPS_EN_REG	SYSEFUSE_REG(0x32c)
#define EFUSE_VQPS_EN	1

#define EFUSE_DATA_0	SYSEFUSE_REG(0x330)

#define EFUSE_PROGRAM_DATA	SYSEFUSE_REG(0x3c0)

#define SYS_EFUSE_CFG		0xb8865000
#define SYS_EFUSE_CTL		0xb8865004
#define EFUSE_DONE	(1 << 0)
#define SYS_EFUSE_CNT		0xb8865008
#define SYS_EFUSE_RDATA 0xb886500c

#define SYS_EFUSE_OUT0 0xb8865010
#define SYS_EFUSE_OUT1 0xb8865014
#define SYS_EFUSE_OUT2 0xb8865018
#define SYS_EFUSE_OUT3 0xb886501c
#define SYS_EFUSE_OUT4 0xb8865020
#define SYS_EFUSE_OUT5 0xb8865024
#define SYS_EFUSE_OUT6 0xb8865028
#define LOAD_AES_KEY	17
#define EFUSE_POW33	16
#define PROGRAM_POW	14
#define EFUSE_PD	13
#define EFUSE_PS	12
#define EFUSE_CSB	11
#define EFUSE_PGENB	10
#define EFUSE_LOAD	9
#define	EFUSE_STROBE	8

#define SYS_MEM_SD		0xb88c0228
#define ONCHIP_MEM_SD	BIT(16)
#define CPU_MEM_SD	BIT(13)
#define SDIO2_MEM_SD	BIT(7)
#define SDIO_MEM_SD	BIT(6)
#define ETH_MEM_SD	BIT(3)
#define CIPHER_MEM_SD	BIT(2)

/*ethernet phy config*/
#define ETHERNET_LDPHY_CFG 0xb8861034
#define LDPHY_SEL			0x80000000
#define RMII_REF_CLK_EN		0x08000000
#define SPD					0x01000000

#define ETHERNET_LDPHY_CFG2	0xb8861038
#define RMII_RX_CLK_INV		0x80000000
#define TX_PHASE_SEL		0x00030000
#define INT_GPIO7			0x3

/*platform configuration*/
#define PLATFORM_CON	0xb8861064
#define ROM_SEL_NOR		(1 << 3)
#define ROM_SEL_NAND	(1 << 2)

/* Values for SPIC_CTRL before relocate */
#define SPICCTRL_DDR_ADDR_BASE	0x82000000
#define SPICCTRL_DDR_ADDR_END	0x820000a0

#define SCKDV			0x8
#define SPIC_BAUD_VALUE		(SCKDV << 0)

#define RD_DUAL_O_CMD		0x3B

#define CS_H_WR_DUM_LEN		0x5
#define CS_H_RD_DUM_LEN		0x0
#define AUCK_DUM_LEN		0x0
#define AUTO_ADDR_LENGTH	0x3
#define RD_DUMMY_LENGTH		0x11
#define SPIC_AUTO_LENGTH_VALUE	((CS_H_WR_DUM_LEN << 28) \
		| (CS_H_RD_DUM_LEN << 26) | (AUCK_DUM_LEN << 18) \
		| (AUTO_ADDR_LENGTH << 16) | RD_DUMMY_LENGTH)

#define WR_BLOCKING		(1 << 9)
#define RD_DUAL_I		(1 << 1)
#define SPIC_VALID_CMD_VALUE	(WR_BLOCKING | RD_DUAL_I)

#define TAG1_VADDR	0xbc060000
#define TAG2_VADDR	0xbc061000
#define TAG3_VADDR	0xbc062000

/*work mode setting*/
#define RST_MODE_ADDR	0xb8860028
#define RST_MASK	0xF
#define NOR	0x0
#define NOR_JTAG0	0x1
#define NOR_SECURE	0x2
#define NOR_ROM_DOWNLOAD	0x3
#define NAND	0x4
#define NAND_JTAG0	0x5
#define NAND_SECURE	0x6
#define NAND_ROM_DOWNLOAD	0x7
#define NOR_JTAG1	0x8
#define NOR_ROM_DEBUG	0xA
#define NAND_ROM_DEBUG	0xB
#define NAND_JTAG1	0xC
#define EMMC	0x0E
#define EMMC_SECURE	0xD
#endif

