#include <common.h>
#include <malloc.h>
#include <net.h>
#include <netdev.h>
#include <spi_flash.h>

#define MAX_SAVEENV_SIZE 512

#define ETH_MACADDR_ID	0x1000001
#define ETH_IPADDR_ID	0x1000002
#define ETH_GATEWAY_ID	0x1000003
#define ETH_NETMASK_ID	0x1000004

/* custom for example */
#define CUSTOM_MODEL_ID	0x5000001

struct hconf {
	char magic[4];
	u32 entry;
	u32 length;
};

struct entry {
	u32 id;
	u32 type_length;
	char value[64];
};

int saveenv_add_entry_to_hconf(struct hconf *hconf, u32 id,
	void *data, u32 data_len)
{
	struct entry entry;
	char *buf = (char *)hconf;

	entry.id = id;
	entry.type_length = data_len << 16;
	memcpy(entry.value, data, data_len);

	/* check buf size */
	if ((hconf->length + 8 + data_len) > MAX_SAVEENV_SIZE)
		return -1;

	memcpy(buf + hconf->length, &entry, data_len+8);
	hconf->entry++;
	hconf->length += 8 + data_len;
	return 0;
}

int do_saveenv(cmd_tbl_t *cmdtp, int flag, int argc,
			char * const argv[])
{
	char *buf;
	char *value;
	int ret;

	buf = malloc(MAX_SAVEENV_SIZE);
	if (buf == NULL)
		return -1;

	/* hconf header */
	struct hconf *hconf = (struct hconf *)buf;

	strncpy(hconf->magic, "noch", 4);
	hconf->entry = 0;
	hconf->length = sizeof(struct hconf);

	value = getenv("ethaddr");
	if (value) {
		uchar mac[6];

		eth_parse_enetaddr(value, mac);

		printf("ethaddr: %s\n", value);
		saveenv_add_entry_to_hconf(hconf, ETH_MACADDR_ID, mac,
			sizeof(mac));
	}

	value = getenv("ipaddr");
	if (value) {
		struct in_addr ip = string_to_ip(value);

		printf("ipaddr: %s\n", value);
		saveenv_add_entry_to_hconf(hconf, ETH_IPADDR_ID, &ip,
			sizeof(ip));
	}

	value = getenv("gateway");
	if (value) {
		struct in_addr ip = string_to_ip(value);

		printf("gateway: %s\n", value);
		saveenv_add_entry_to_hconf(hconf, ETH_GATEWAY_ID, &ip,
			sizeof(ip));
	}

	value = getenv("netmask");
	if (value) {
		struct in_addr ip = string_to_ip(value);

		printf("netmask: %s\n", value);
		saveenv_add_entry_to_hconf(hconf, ETH_NETMASK_ID, &ip,
			sizeof(ip));
	}

	/* save custom model */
	value = getenv("model");
	if (value) {

		printf("model: %s\n", value);
		saveenv_add_entry_to_hconf(hconf, CUSTOM_MODEL_ID, value,
			strlen(value));
	}
#ifdef CONFIG_RTS_QSPI
#ifdef RTS_MTD_HCONF_OFFSET
	/* update to flash, in cmd_sf.c */
	ret = spi_flash_update_external(RTS_MTD_HCONF_OFFSET,
		hconf->length, buf);

	if (ret == 0)
		printf("OK\n");
#else
	printf("saveenv: no HCONF define!\n");
#endif
#endif
#ifdef CONFIG_RTS_SPI_NAND_FLASH
#ifdef RTS_MTD_HCONF_OFFSET
		ret = update_image_to_nand(RTS_MTD_HCONF_OFFSET,
				hconf->length, (unsigned char *)buf);

	if (ret == 0)
		printf("OK\n");
#else
	printf("saveenv: no HCONF define!\n");
#endif
#endif
	free(buf);
	return ret;
}



#define SET_SAVENV_HELP

U_BOOT_CMD(
		saveenv, 1,	1, do_saveenv,
		"save environment variables to flash",
		""
		SET_SAVENV_HELP
);
