#include <common.h>
#include <malloc.h>
#include <asm/io.h>
#include <net.h>
#include <spi_flash.h>
/*#include <configs/rlxboard.h>*/
#include <part.h>
#include <fat.h>
#include <fs.h>
#include <mmc.h>

#ifndef _MEM_TEST3_
static char *tmpfile;
#define GETMEM(addr)	(*(volatile u32 *)(addr))

static u32 _do_get_file_(char *filename)
{
	u32 len;

	tmpfile = filename;

	/*setenv("bootfile", tmpfile);*/
	if  (net_loop(TFTPSRV) < 0)
		return 0;

	len = getenv_hex("filesize", 0);

	setenv("filesize", NULL);
	return len;
}

static int _do_write_file_(u32 offset, u32 len, u32 loadaddr,
				u32 partition_size, u32 magic_num)
{
	int ret = 0;
#ifdef CONFIG_RTS_EMMC_BOOT
	struct mmc *mmc;
	int n;
	lbaint_t blkcnt;
#ifdef CONFIG_FLASH_START
	char *buf;

	buf = (char *)loadaddr;
#endif
#else
	char *buf;

	buf = (char *)loadaddr;
#endif

	if (len == 0)
		return CMD_RET_FAILURE;
	if (partition_size == 0)
		partition_size = len;

#if (defined CONFIG_RTS_QSPI) && (!defined CONFIG_RTS_EMMC_BOOT)
	if ((get_rst_mode() == NOR_MODE) ||
#ifdef CONFIG_BOARD_RTS3915
		(get_rst_mode() == NOR_SEC_MODE) ||
#endif
		(get_rst_mode() == NOR_RES_MODE))
	ret = spi_flash_update_external(offset, len, buf);
#endif

#ifdef CONFIG_RTS_SPI_NAND_FLASH
	if ((get_rst_mode() == NAND_MODE) ||
#ifdef CONFIG_BOARD_RTS3915
		(get_rst_mode() == NAND_SEC_MODE) ||
#endif
		(get_rst_mode() == NAND_RES_MODE))
		ret = for_update_image_to_nand(offset, len, (unsigned char *)buf,
					partition_size);
#endif

#ifdef CONFIG_RTS_EMMC_BOOT
	mmc = find_mmc_device(0);
	if ((strcmp(tmpfile, "/u-boot.bin") == 0) || magic_num == 0x626f6f74) {
#ifdef CONFIG_FLASH_START
		printf("tmpfile is %s\n", tmpfile);
		ret = spi_flash_update_external(offset, len, buf);
		return ret;
#else
		blk_select_hwpart_devnum(IF_TYPE_MMC, 0, 1);
		mmc_set_part_conf(mmc, 0, 1, 1);
#endif
	} else {
		blk_select_hwpart_devnum(IF_TYPE_MMC, 0, 0);
		mmc_set_part_conf(mmc, 0, 0, 0);
	}

	if (mmc_getwp(mmc) == 1) {
		printf("Error: card is write protected!\n");
		return -1;
	}
	if ((len % 512) == 0)
		blkcnt = len >> 9;
	else
		blkcnt = (len >> 9) + 1;
	printf("start is %x, cnt is %x\n", (offset >> 9), blkcnt);
	blk_derase(mmc_get_blk_desc(mmc), (offset >> 9), blkcnt);
	n = blk_dwrite(mmc_get_blk_desc(mmc), (offset >> 9),
		       blkcnt, (void *)loadaddr);

	if (n == blkcnt)
		ret = 0;
	else
		ret = -1;

	blk_select_hwpart_devnum(IF_TYPE_MMC, 0, 1);
	mmc_set_part_conf(mmc, 0, 1, 1);
#endif

	setenv("fileaddr", NULL);
	return ret;
}

static int do_update_uboot(void)
{
	int ret = 0;
	u32 len = 0;
	u32 offset = 0;
	u32 loadaddr = CONFIG_LOADADDR;

	len = _do_get_file_("/u-boot.bin");

	ret = _do_write_file_(offset, len, loadaddr, 0, 0);

	debug("get %s from %x , write %x byte to offset %x\n", tmpfile, loadaddr, len, offset);
	return ret;
}

#ifdef CONFIG_CRYPTO_UPDATE
static int patchkernelimg(u32 addr, u32 len) {

	u32 dlen;
	u32 crc;
	dlen = image_get_data_size((void *)addr);
	dlen = (((dlen + 15)  >> 4) << 4);
	len += dlen - image_get_data_size((void *)addr);

	rlx_aes_ecb_encrypt((void *)(addr + 64), (void *)(addr + 64), dlen);

	crc = crc32_wd(0, (void *)(addr + 64), dlen, CHUNKSZ_CRC32);
	image_set_dcrc((void *)addr, crc);
	image_set_size((void *)addr, dlen);

	image_set_hcrc((void *)addr, 0);
	crc = crc32(0, (void *)addr, 64);
	image_set_hcrc((void *)addr, crc);
	return len;
}
#endif

static int do_update_kernel(void)
{
	int ret = 0;
	u32 offset = SPI_KERNEL_OFFSET;
	u32 len = 0;
	u32 loadaddr = CONFIG_LOADADDR;

	len = _do_get_file_("/vmlinux.img");
#ifdef CONFIG_CRYPTO_UPDATE
	len = patchkernelimg(loadaddr, len);
#endif
	ret = _do_write_file_(offset, len, loadaddr, 0, 0);

	debug("get %s from %x , write %x byte to offset %x\n", tmpfile, loadaddr, len, offset);
	return ret;
}

#ifdef SPI_ROOTFS_OFFSET
static int do_update_rootfs(void)
{
	int ret = 0;
	u32 offset = SPI_ROOTFS_OFFSET;
	u32 len = 0;
	u32 loadaddr = CONFIG_LOADADDR;
	u32 partition_size = SPI_ROOTFS_SIZE;

	len = _do_get_file_("/rootfs.bin");
	ret = _do_write_file_(offset, len, loadaddr, partition_size, 0);

	debug("get %s from %x , write %x byte to offset %x\n", tmpfile, loadaddr, len, offset);
	return ret;
}
#endif
/*
static int do_update_ldctable(void)
{
	int ret = 0;
	u32 offset = SPI_LDC_OFFSET ;
	u32 len = 0;
	u32 loadaddr = CONFIG_LOADADDR;

	len = _do_get_file_("/ldc.bin");
	ret = _do_write_file_(offset, len, loadaddr, 0);

	debug("get %s from %x , write %x byte to offset %x\n", tmpfile, loadaddr, len, offset);
	return ret;
}
*/
static int _do_write_all(u32 all_len)
{
	int ret = 0;

	u32 magic_num;
	u32 ram_offset = 0x100;
	u32 flash_offset = 0;
	u32 filelen;
	u32 partition_size = 0;
#ifdef CONFIG_CRYPTO_UPDATE
	u32 klen;
	u32 addr;
	u32 all_len_temp;

	all_len_temp = all_len;
#endif

	all_len -= 0x100;
	while (all_len != 0) {

		magic_num = htonl(GETMEM(CONFIG_LOADADDR + ram_offset));
		debug("%x:%x\n", (CONFIG_LOADADDR + ram_offset), magic_num);
		partition_size = htonl(GETMEM(CONFIG_LOADADDR + ram_offset + 0x0C));
		flash_offset = htonl(GETMEM(CONFIG_LOADADDR + ram_offset + 0x14));
		filelen = htonl(GETMEM(CONFIG_LOADADDR + ram_offset + 0x1C));

		switch (magic_num) {
		/*uboot*/
		case 0x626f6f74:

		/*MCU fw*/
		case 0x6669726d:

		/*hw config*/
		case 0x68617264:

		/*sw config*/
		case 0x6a667332:

		/*kernel*/
		case 0x6c696e78:

		/*rootfs*/
		case 0x726f6f74:

		/*ldc table*/
		case 0x6c646300:

		/*gpt head*/
		case 0x67707468:

		/*gpt tail*/
		case 0x67707474:
			printf("offset is %x\n", flash_offset);
			break;

		/*file end*/
		case 0x46454f46:
			printf("file end\n");
			return 0;

		default:
			printf("user defined magic number %x at %x\n",
			magic_num, (CONFIG_LOADADDR + ram_offset));
			break;
		}
#ifdef CONFIG_CRYPTO_UPDATE
		if (flash_offset == SPI_KERNEL_OFFSET) {
			addr = CONFIG_LOADADDR + all_len_temp;
			addr = (addr + 0x3) & ~0x3;
			memcpy((void *)addr,
			(void *)(CONFIG_LOADADDR + ram_offset + 0x20), filelen);
			klen = patchkernelimg(addr, filelen);
			ret = _do_write_file_(flash_offset, klen, addr,
				partition_size, magic_num);
		}
		else
#endif
			ret = _do_write_file_(flash_offset, filelen,
					(CONFIG_LOADADDR + ram_offset + 0x20),
					partition_size, magic_num);

		if (ret != 0)
			return ret;
		printf("get %x file from %x, wtite %x bytes to flash offset %x\n", magic_num, ram_offset, filelen, flash_offset);
		all_len -= (filelen + 0x24);
		ram_offset += (filelen + 0x24);
	}
	debug("update all image from ethernet and write to flash\n");
	return ret;
}
/*
static int do_update_flash(void)
{
	int ret = 0;
	u32 all_len = 0;

	all_len = getenv_hex("filesize", 0);

	ret = _do_write_all(all_len);

	return ret;
}
*/
static int do_update_all(void)
{
	int ret = 0;
	u32 all_len = 0;

	all_len = _do_get_file_("/linux.bin");

	if (all_len == 0)
		return CMD_RET_FAILURE;

	ret = _do_write_all(all_len);

	return ret;
}

#ifdef CONFIG_LCD
static int do_update_lcdimage(void)
{
	int ret = 0;
	u32 offset = SPI_LOGO_OFFSET;
	u32 len = 0;
	u32 loadaddr = CONFIG_LOADADDR;

	len = _do_get_file_("/logo.bmp");
	if (len == 0)
		return CMD_RET_FAILURE;
	ret = _do_write_file_(offset, len, loadaddr, 0, 0);

	debug("get %s from %x , write %x byte to offset %x\n", tmpfile, loadaddr,
		  len, offset);

	return ret;
}
#endif

static int do_update(cmd_tbl_t *cmdtp, int flag, int argc,
			char * const argv[])
{

	const char *cmd;
	int ret = CMD_RET_FAILURE;

	/* need at least two arguments */
	if (argc < 2)
		goto usage;

	cmd = argv[1];
	--argc;
	++argv;

	if (strcmp(cmd, "uboot") == 0) {
		ret = do_update_uboot();
		goto done;
	}

	if (strcmp(cmd, "kernel") == 0) {
		ret = do_update_kernel();
		goto done;
	}

#ifdef SPI_ROOTFS_OFFSET
	if (strcmp(cmd, "rootfs") == 0) {
		ret = do_update_rootfs();
		goto done;
	}
#endif

	if (strcmp(cmd, "all") == 0) {
		ret = do_update_all();
		goto done;
	}

#ifdef CONFIG_LCD
	if (strcmp(cmd, "lcdimage") == 0) {
		ret = do_update_lcdimage();
		goto done;
	}
#endif

done:
	if (ret == CMD_RET_SUCCESS)
		printf("update done\n\n");
	else
		printf("update failed\n\n");

	return ret;

usage:
	return CMD_RET_USAGE;

}

int load_bin_from_sd(void)
{
	unsigned long addr;
	const char *addr_str;
	const char *filename;
	unsigned long bytes;
	unsigned long pos;
	int len_read;
	unsigned long time;
	int ret;
	int *temp = &len_read;

	if (fs_set_blk_dev("mmc", "0:1", FS_TYPE_FAT))
		if (fs_set_blk_dev("mmc", "0:0", FS_TYPE_FAT))
			return 0;

	addr_str = getenv("loadaddr");
	if (addr_str != NULL)
		addr = simple_strtoul(addr_str, NULL, 16);
	else
		addr = CONFIG_SYS_LOAD_ADDR;

	filename = "linux.bin";

	bytes = 0;
	pos = 0;
	time = get_timer(0);
	ret = fs_read(filename, addr, pos, bytes, (loff_t *)temp);
	printf("in load_bin_from_sd\n");
	time = get_timer(time);
	if (ret < 0)
		return 0;

	printf("%d bytes read in %lu ms", len_read, time);
	if (time > 0) {
		puts(" (");
		print_size(len_read / time * 1000, "/s");
		puts(")");
	}
	puts("\n");

	_do_write_all(len_read);

	return 0;
}

int load_key_from_sd(void)
{
	unsigned long addr;
	const char *filename;
	unsigned long bytes;
	unsigned long pos;
	int len_read;
	unsigned long time;
	int ret;
	int *temp = &len_read;

	if (fs_set_blk_dev("mmc", "0:1", FS_TYPE_FAT))
		if (fs_set_blk_dev("mmc", "0:0", FS_TYPE_FAT))
			return 0;

	addr = CONFIG_LOADADDR;
	printf("load key from sd addr %lx\n", addr);

	filename = "ipcam_crypto.bin";

	bytes = 0;
	pos = 0;
	time = get_timer(0);
	ret = fs_read(filename, addr, pos, bytes, (loff_t *)temp);
	printf("in load_key_from_sd\n");
	time = get_timer(time);
	if (ret < 0)
		return 0;

	printf("%d bytes read in %lu ms", len_read, time);
	if (time > 0) {
		puts(" (");
		print_size(len_read / time * 1000, "/s");
		puts(")");
	}
	puts("\n");

	return 0;
}

int load_iv_from_sd(void)
{
	unsigned long addr;
	const char *filename;
	unsigned long bytes;
	unsigned long pos;
	int len_read;
	unsigned long time;
	int ret;
	int *temp = &len_read;

	if (fs_set_blk_dev("mmc", "0:1", FS_TYPE_FAT))
		if (fs_set_blk_dev("mmc", "0:0", FS_TYPE_FAT))
			return 0;

	addr = CONFIG_LOADADDR;
	printf("load iv from sd addr %lx\n", addr);

	filename = "ipcam_crypto_iv.bin";

	bytes = 0;
	pos = 0;
	time = get_timer(0);
	ret = fs_read(filename, addr, pos, bytes, (loff_t *)temp);
	printf("in load_iv_from_sd\n");
	time = get_timer(time);
	if (ret < 0)
		return 0;

	printf("%d bytes read in %lu ms", len_read, time);
	if (time > 0) {
		puts(" (");
		print_size(len_read / time * 1000, "/s");
		puts(")");
	}
	puts("\n");

	return 0;
}

#ifdef CONFIG_CMD_SF_TEST
#define UPDATE_TEST_HELP "\nsf test offset len		" \
		"- run a very basic destructive test"
#else
#define UPDATE_TEST_HELP
#endif

U_BOOT_CMD(
	update,	5,	1,	do_update,
	"update xxx",
	"uboot	- get uboot.bin by tftp and write to flash\n"
	"update kernel	- get kernel image by tftp and write to flash\n"
#ifdef RTS_MTD_ROOTFS_OFFSET
	"update rootfs	- get rootfs by tftp and write to flash\n"
#endif
	"update all	- get all image by tftp , parse it and  write to flash\n"
	UPDATE_TEST_HELP
);
#endif
