/*
 *  Copyright (C) 2002 Thomas Gleixner (tglx@linutronix.de)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <common.h>
#include <linux/mtd/nand.h>
#include <linux/sizes.h>
#include <mtd/rts_spinand.h>

#define LP_OPTIONS NAND_SAMSUNG_LP_OPTIONS
#define LP_OPTIONS16 (LP_OPTIONS | NAND_BUSWIDTH_16)

#define SP_OPTIONS NAND_NEED_READRDY
#define SP_OPTIONS16 (SP_OPTIONS | NAND_BUSWIDTH_16)

/*
 * The chip ID list:
 *    name, device ID, page size, chip size in MiB, eraseblock size, options
 *
 * If page size and eraseblock size are 0, the sizes are taken from the
 * extended chip ID.
 */
struct nand_flash_dev nand_flash_ids[] = {
	/*
	 * Some incompatible NAND chips share device ID's and so must be
	 * listed by full ID. We list them first so that we can easily identify
	 * the most specific match.
	 */
#ifdef CONFIG_RTS_SPI_NAND_FLASH
#ifdef CONFIG_NAND_QUAD_CHANNEL
	{"MX35LF1GE4AB",
		{ .id = {0xc2, 0x12, 0xc2} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"MX35LF2GE4AB",
		{ .id = {0xc2, 0x22, 0xc2} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_2|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"MX35LF1G24AD",
		{ .id = {0xc2, 0x14, 0x03} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"MX35LF2G24AD",
		{ .id = {0xc2, 0x24, 0x03} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_2|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"W25N01GVZE1G",
		{ .id = {0xef, 0xaa, 0x21} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_2,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"W25M02GVZE1G",
		{ .id = {0xef, 0xab, 0x21} },
		  SZ_2K, SZ_256, SZ_128K, 0, OTP_TYPE_2|SW_DIE_WITH2,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F1GQ4UBYIG",
		{ .id = {0xc8, 0xd1, 0xc8} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F2GQ4UBYIG",
		{ .id = {0xc8, 0xd2, 0xc8} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F1GQ4UEYIH",
		{ .id = {0xc8, 0xd9, 0xc8} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F2GQ5UEYIG",
		{ .id = {0xc8, 0x52, 0xc8} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F1GQ5UEYIG",
		{ .id = {0xc8, 0x51, 0xc8} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"XT26G01B",
		{ .id = {0x0b, 0xf1, 0x0b} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x32,
		  ADDR_SINGLE_CH|DATA_QUAD_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"XT26G02B",
		{ .id = {0x0b, 0xf2, 0x0b} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x03, ADDR_SINGLE_CH|DATA_SINGLE_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"XT26G04A",
		{ .id = {0x0b, 0xe3, 0x0b} },
		  SZ_2K, SZ_512, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"FM25S01A",
		{ .id = {0xa1, 0xe4, 0x7f} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"FM25S02A",
		{ .id = {0xa1, 0xe5, 0xa1} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"F50L1G41LB",
		{ .id = {0xc8, 0x01, 0x7f} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x6B, ADDR_SINGLE_CH|DATA_QUAD_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
#endif
#ifdef CONFIG_NAND_DUAL_CHANNEL
	{"MX35LF1GE4AB",
		{ .id = {0xc2, 0x12, 0xc2} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"MX35LF2GE4AB",
		{ .id = {0xc2, 0x22, 0xc2} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_2|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"MX35LF1G24AD",
		{ .id = {0xc2, 0x14, 0x03} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"MX35LF2G24AD",
		{ .id = {0xc2, 0x24, 0x03} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_2|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"W25N01GVZE1G",
		{ .id = {0xef, 0xaa, 0x21} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_2,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"W25M02GVZE1G",
		{ .id = {0xef, 0xab, 0x21} },
		  SZ_2K, SZ_256, SZ_128K, 0, OTP_TYPE_2|SW_DIE_WITH2,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F1GQ4UBYIG",
		{ .id = {0xc8, 0xd1, 0xc8} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F2GQ4UBYIG",
		{ .id = {0xc8, 0xd2, 0xc8} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F1GQ4UEYIHY",
		{ .id = {0xc8, 0xd9, 0xc8} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F2GQ5UEYIG",
		{ .id = {0xc8, 0x52, 0xc8} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"GD5F1GQ5UEYIG",
		{ .id = {0xc8, 0x51, 0xc8} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"XT26G01B",
		{ .id = {0x0b, 0xf1, 0x0b} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"XT26G02B",
		{ .id = {0x0b, 0xf2, 0x0b} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x03, ADDR_SINGLE_CH|DATA_SINGLE_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"XT26G04A",
		{ .id = {0x0b, 0xe3, 0x0b} },
		  SZ_2K, SZ_512, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"FM25S01A",
		{ .id = {0xa1, 0xe4, 0x7f} },
		  SZ_2K, SZ_128, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"FM25S02A",
		{ .id = {0xa1, 0xe5, 0xa1} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
	{"F50L1G41LB",
		{ .id = {0xc8, 0x01, 0x7f} },
		  SZ_2K, SZ_256, SZ_128K, 0, PLANE_WITH_1|OTP_TYPE_1,
		  0x3B, ADDR_SINGLE_CH|DATA_DUAL_CH, 0x8, 0x02,
		  ADDR_SINGLE_CH|DATA_SINGLE_CH,
		  0, 8, 64, NAND_ECC_INFO(1, SZ_512),
		  2 },
#endif
#endif
	{NULL}
};

/* Manufacturer IDs */
struct nand_manufacturers nand_manuf_ids[] = {
	{NAND_MFR_TOSHIBA, "Toshiba"},
	{NAND_MFR_SAMSUNG, "Samsung"},
	{NAND_MFR_FUJITSU, "Fujitsu"},
	{NAND_MFR_NATIONAL, "National"},
	{NAND_MFR_RENESAS, "Renesas"},
	{NAND_MFR_STMICRO, "ST Micro"},
	{NAND_MFR_HYNIX, "Hynix"},
	{NAND_MFR_MICRON, "Micron"},
	{NAND_MFR_AMD, "AMD/Spansion"},
	{NAND_MFR_MACRONIX, "Macronix"},
	{NAND_MFR_EON, "Eon"},
	{NAND_MFR_SANDISK, "SanDisk"},
	{NAND_MFR_INTEL, "Intel"},
	{NAND_MFR_ATO, "ATO"},
	{NAND_MFR_WINBOND, "Winbond"},
	{NAND_MFR_GIGADEVICE, "GigaDevice"},
	{NAND_MFR_XTX, "XTX"},
	{NAND_MFR_FUDAN, "Fudan"},
	{NAND_MFR_ESMT, "ESMT"},
	{0x0, "Unknown"}
};

EXPORT_SYMBOL(nand_manuf_ids);
EXPORT_SYMBOL(nand_flash_ids);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Thomas Gleixner <tglx@linutronix.de>");
MODULE_DESCRIPTION("Nand device & manufacturer IDs");
