/*
 * SPI flash interface
 *
 * Copyright (C) 2008 Atmel Corporation
 * Copyright (C) 2010 Reinhard Meyer, EMK Elektronik
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <spi.h>

static int spi_flash_read_write(struct spi_slave *spi,
				const u8 *cmd, size_t cmd_len,
				const u8 *data_out, u8 *data_in,
				size_t data_len)
{
	unsigned long flags = SPI_XFER_BEGIN;
	int ret;

#ifdef CONFIG_SF_DUAL_FLASH
	if (spi->flags & SPI_XFER_U_PAGE)
		flags |= SPI_XFER_U_PAGE;
#endif
	if (data_len == 0)
		flags |= SPI_XFER_END;

	ret = spi_xfer(spi, cmd_len * 8, cmd, NULL, flags);
	if (ret) {
		debug("SF: Failed to send command (%zu bytes): %d\n",
		      cmd_len, ret);
	} else if (data_len != 0) {
		ret = spi_xfer(spi, data_len * 8, data_out, data_in,
					SPI_XFER_END);
		if (ret)
			debug("SF: Failed to transfer %zu bytes of data: %d\n",
			      data_len, ret);
	}

	return ret;
}

int spi_flash_cmd_read(struct spi_slave *spi, const u8 *cmd,
		size_t cmd_len, void *data, size_t data_len)
{
	return spi_flash_read_write(spi, cmd, cmd_len, NULL, data, data_len);
}

int spi_flash_cmd(struct spi_slave *spi, u8 cmd, void *response, size_t len)
{
	return spi_flash_cmd_read(spi, &cmd, 1, response, len);
}

int spi_flash_cmd_write(struct spi_slave *spi, const u8 *cmd, size_t cmd_len,
		const void *data, size_t data_len)
{
	return spi_flash_read_write(spi, cmd, cmd_len, data, NULL, data_len);
}

int spi_flash_enable_4B_mode(struct spi_slave *spi)
{
	return flash_enable_4B(spi);
}

int spi_flash_set_4B_AutoRead(struct spi_slave *spi)
{
	return spi_flash_set_AutoRead_AddrLen(spi, 4);
}

int spi_flash_set_reset_init(struct spi_slave *spi)
{
	return spi_flash_set_reset(spi);
}

int spi_flash_set_spic_init(struct spi_slave *spi, u8 dummy_byte, u8 read_cmd)
{
	return spi_flash_set_spic(spi, dummy_byte, read_cmd);
}

int spi_flash_set_auto_mode(struct spi_slave *spi)
{
	return spi_flash_set_auto(spi);
}

int spi_flash_enable_qpi_ops(struct spi_slave *spi)
{
	return flash_enable_qpi(spi);
}

int spi_flash_flush_rst_fifo(struct spi_slave *spi)
{
	return flash_flush_rst_fifo(spi);
}

int spi_flash_set_rst_fifo(struct spi_slave *spi, u16 reset_flow)
{
	return flash_set_rst_fifo(spi, reset_flow);
}

int spi_flash_exit_QPI(struct spi_slave *spi)
{
	return flash_exit_qpi(spi);
}

int spi_flash_exit_prm(struct spi_slave *spi)
{
	return flash_exit_prm(spi);
}

int spi_flash_disable_prm(struct spi_slave *spi)
{
	return flash_disable_prm(spi);
}

int spi_flash_enable_prm(struct spi_slave *spi)
{
	return flash_enable_prm(spi);
}

int spi_flash_set_prm_mode(struct spi_slave *spi)
{
	return spi_flash_set_prm(spi);
}
