/*
 * RTS LCD Controller
 *
 * (C) Copyright 2001-2002
 * Wolfgang Denk, DENX Software Engineering -- wd@denx.de
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

/************************************************************************/
/* ** HEADER FILES							*/
/************************************************************************/

/* #define DEBUG */
#include <common.h>
#include <asm/io.h>
#include <lcd.h>
#include <linux/types.h>
#include <stdarg.h>
#include <stdio_dev.h>
#include "rts_fb.h"

#ifndef REG32
#define REG32(reg)		(*(volatile u32 *)(reg))
#endif
#define lcdc_read(addr)		REG32(addr)
#define lcdc_write(addr, val)	REG32(addr) = (val)
#define RTS_GETFIELD(val, width, offset)	\
			((val >> offset) & ((1 << width) - 1))
#define RTS_SETFIELD(reg, field, width, offset)	((reg & \
			(~(((1 << width) - 1) <<	offset))) \
			| ((field & ((1 << width) - 1)) << offset))
#define clear_bit(addr, val)	lcdc_write((addr), (lcdc_read(addr) & ~(val)))
#define set_bit(addr, val)	lcdc_write((addr), (lcdc_read(addr) | (val)))

#ifdef CONFIG_LCD
/*----------------------------------------------------------------------*/
/*
 * Define panel_info video struct for your display.
 */

#ifdef CONFIG_TIANMA050
/* LCD outputs connected to a video DAC  */
#define LCD_BPP	LCD_COLOR16

/* 800x480x16 @ 60 Hz */
vidinfo_t panel_info = {
	.vl_col		= 800,
	.vl_row		= 480,
	.vl_clkp	= 1,
	.vl_hsp		= 1,
	.vl_vsp		= 1,
	.vl_dp		= 1,
	.rgb_outctrl = 3,
	.vl_format	= FORMAT_RGB_565,
	.vl_bpix	= LCD_BPP,
	.hfp		= 13,
	.hbp		= 29,
	.hsw		= 48,
	.vfp		= 13,
	.vbp		= 29,
	.vsw		= 3,
	.pxl_clk	= 30000000,
	.mode		= RGB_INTERFACE,
	.bgcolor	= 0xff0000,
};
#endif /* CONFIG_TIANMA050 */

#ifdef CONFIG_RFH_495B
/* LCD outputs connected to a video DAC  */
#define LCD_BPP	LCD_COLOR32

/* 480x854x16 @ 60 Hz */
vidinfo_t panel_info = {
	.vl_col		= 480,
	.vl_row		= 854,
	.vl_clkp	= 0,
	.vl_hsp		= 1,
	.vl_vsp		= 1,
	.vl_dp		= 1,
	.rgb_outctrl = 7,
	.vl_format	= FORMAT_ARGB_8888,
	.vl_bpix	= LCD_BPP,
	.hfp		= 50,
	.hbp		= 40,
	.hsw		= 8,
	.vfp		= 4,
	.vbp		= 16,
	.vsw		= 10,
	.pxl_clk	= 30000000,
	.mode		= RGB_INTERFACE,
	.bgcolor	= 0xff0000,
};
#endif /* CONFIG_RFH_495B */

#ifdef CONFIG_STARTEK_KD050WVFLA037
/* LCD outputs connected to a video DAC  */
#define LCD_BPP LCD_COLOR32
/* 800x480x16 @ 60 Hz */
vidinfo_t panel_info = {
	.vl_col		= 800,
	.vl_row		= 480,
	.vl_clkp	= 0,
	.vl_hsp		= 1,
	.vl_vsp		= 1,
	.vl_dp		= 1,
	.vl_format	= FORMAT_ARGB_8888,
	.vl_bpix	= LCD_BPP,
	.hfp		= 8,
	.hbp		= 4,
	.hsw		= 4,
	.vfp		= 8,
	.vbp		= 4,
	.vsw		= 4,
	.pxl_clk	= 30000000,
	.mode		= LVDS_INTERFACE,
	.lvds_format = LVDS_8BIT_VESA,
	.bgcolor	= 0xff0000,
};
#endif /* CONFIG_STARTEK_KD050WVFLA037 */

#ifdef CONFIG_TIANMA_TM050RDH03
/* LCD outputs connected to a video DAC  */
#define LCD_BPP LCD_COLOR32
/* 800x480x16 @ 61 Hz */
vidinfo_t panel_info = {
	.vl_col		= 800,
	.vl_row		= 480,
	.vl_clkp	= 1,
	.vl_hsp		= 1,
	.vl_vsp		= 1,
	.vl_dp		= 1,
	.rgb_outctrl = 7,
	.vl_format	= FORMAT_ARGB_8888,
	.vl_bpix	= LCD_BPP,
	.hfp		= 40,
	.hbp		= 40,
	.hsw		= 48,
	.vfp		= 13,
	.vbp		= 29,
	.vsw		= 3,
	.pxl_clk	= 30000000,
	.mode		= RGB_INTERFACE,
	.bgcolor	= 0xff0000,
};
#endif /* CONFIG_TIANMA_TM050RDH03 */

#ifdef CONFIG_TIANMA_TM101JDHP01
#define LCD_BPP LCD_COLOR16
/* 800x480x16 @ 61 Hz */
vidinfo_t panel_info = {
	.vl_col		= 1280,
	.vl_row		= 800,
	.vl_clkp	= 0,
	.vl_hsp		= 1,
	.vl_vsp		= 1,
	.vl_dp		= 1,
	.vl_format	= FORMAT_ARGB_8888,
	.vl_bpix	= LCD_BPP,
	.hfp		= 64,
	.hbp		= 4,
	.hsw		= 1,
	.vfp		= 40,
	.vbp		= 1,
	.vsw		= 1,
	.pxl_clk	= 70000000,
	.mode		= LVDS_INTERFACE,
	.lvds_format = LVDS_8BIT_VESA,
	.bgcolor	= 0xff0000,
};
#endif /* CONFIG_TIANMA_TM101JDHP01 */

/*----------------------------------------------------------------------*/

static int rtsfb_init_mem(void *lcdbase, vidinfo_t *vid);
static void rtsfb_setup_gpio(vidinfo_t *vid);
static void rtsfb_enable_controller(vidinfo_t *vid);
static int rtsfb_init(vidinfo_t *vid);
static void set_gpio_spi(void);

static inline void lcdc_writereg_field(u32 addr, u32 val,
				u32 off, u32 width)
{
	u32 v;

	v = lcdc_read(addr);
	v = RTS_SETFIELD(v, val, width, off);

	lcdc_write(addr, v);
}

/************************************************************************/
/* ---------------  RTS chipset specific functions  ------------------- */
/************************************************************************/
ushort *configuration_get_cmap(void)
{
	struct rtsfb_info *fbi = &panel_info.rts;

	return (ushort *)fbi->palette;
}

void lcd_ctrl_init(void *lcdbase)
{
	rtsfb_init_mem(lcdbase, &panel_info);
	#ifdef CONFIG_RFH_495B
	set_gpio_spi();
	#endif
	rtsfb_init(&panel_info);
	rtsfb_enable_controller(&panel_info);
}

/*----------------------------------------------------------------------*/
static void fb_setcolreg(unsigned int regno, unsigned int red,
			   unsigned int green, unsigned int blue, unsigned int transp)
{
	struct rtsfb_info *fbi = &panel_info.rts;
	unsigned short *palette = (unsigned short *)fbi->palette;
	unsigned int val;
	u_int htransp = 0xffff;

	if (transp)
		htransp = transp;

	if (regno < fbi->palette_size) {
		val = ((htransp & 0xff00) << 16);
		val |= ((red & 0xff00) << 8);
		val |= ((green & 0xff00) << 0);
		val |= ((blue & 0xff00) >> 8);
	}

	debug("setcolreg: reg %2d : R=%02X G=%02X B=%02X VAL=%08x\n",
		  regno, red, green, blue, val);
	lcdc_write(LCDC_ARGB_LOOKUP_TABLE + (regno << 2), val);
}

/*----------------------------------------------------------------------*/
void lcd_setcolreg(void)
{
	int i;
	unsigned int red, green, blue, transp;
	int format = panel_info.vl_format;

	/* Setting the palette */
	if (format == FORMAT_RGB_565) {
		for (i = 0; i < 64; i++) {
			red = i << 11;
			green = i << 10;
			blue = i << 11;
			transp = 0;
			fb_setcolreg(i, red, green, blue, transp);
		}
	} else if (format == FORMAT_ARGB_1555) {
		for (i = 0; i < 32; i++) {
			red = i << 11;
			green = i << 11;
			blue = i << 11;
			transp = 0;
			fb_setcolreg(i, red, green, blue, transp);
		}
	} else if (format == FORMAT_ARGB_4444) {
		for (i = 0; i < 16; i++) {
			red = i << 12;
			green = i << 12;
			blue = i << 12;
			transp = i << 12;
			fb_setcolreg(i, red, green, blue, transp);
		}
	} else {
		for (i = 0; i < 256; i++) {
			red = i << 8;
			green = i << 8;
			blue = i << 8;
			transp = i << 8;
			fb_setcolreg(i, red, green, blue, transp);
		}
	}
}

/*----------------------------------------------------------------------*/
int lcd_get_size(int *line_length)
{
	*line_length = (panel_info.vl_col * NBITS(panel_info.vl_bpix)) / 8;
	return *line_length * panel_info.vl_row;
}

/*----------------------------------------------------------------------*/
void lcd_enable(void)
{
	vidinfo_t *vid = &panel_info;
	struct rtsfb_info *fbi = &vid->rts;
	u32 t;

	/* LCD Controller Configures */
	debug("vid: vl_col=%d hslen=%d lm=%d rm=%d\n",
		vid->vl_col, vid->hsw,
		vid->hfp, vid->hbp);
	debug("vid: vl_row=%d vslen=%d um=%d bm=%d\n",
		vid->vl_row, vid->vsw,
		vid->vfp, vid->vbp);

	lcdc_writereg_field(LCDC_DDR_GRAPHIC_OFFSET,
		0, 16, 12);
	lcdc_writereg_field(LCDC_DDR_GRAPHIC_OFFSET,
		0, 0, 12);
	lcdc_writereg_field(LCDC_DDR_GRAPHIC_SIZE,
		vid->vl_col, 16, 12);
	lcdc_writereg_field(LCDC_DDR_GRAPHIC_SIZE,
		vid->vl_row, 0, 12);

	lcdc_writereg_field(LCDC_DISPLAY_GRAPHIC_OFFSET,
		0, 16, 12);
	lcdc_writereg_field(LCDC_DISPLAY_GRAPHIC_OFFSET,
		0, 0, 12);
	lcdc_writereg_field(LCDC_DISPLAY_GRAPHIC_SIZE,
		vid->vl_col, 16, 12);
	lcdc_writereg_field(LCDC_DISPLAY_GRAPHIC_SIZE,
		vid->vl_row, 0, 12);

	lcdc_write(LCDC_REG_LOAD, 1);

	fbi->screen &= 0x1fffffff;
	lcdc_write(LCDC_GRAPHIC_ADDR_DDR_SET, fbi->screen);
	debug("fbi->screen ddr addr %p\n", fbi->screen);
	t = fbi->screen + vid->vl_col * vid->vl_row;
	lcdc_write(LCDC_GRAPHIC_ADDR_DDR_SET_UV, t);
	lcdc_write(LCDC_GRAPHIC_ADDR_DDR_SET_VLD, 1);
	/* graphic enable */
	lcdc_writereg_field(LCDC_FUNC, 1, 4, 1);
	printf("graphic enabled\n");
}

/************************************************************************/
/* ** RTS255 specific routines						*/
/************************************************************************/

/*
 * Calculate fb size for VIDEOLFB_ATAG. Size returned contains fb,
 * descriptors and palette areas.
 */
ulong calc_fbsize(void)
{
	ulong size;
	int line_length;

	line_length = (panel_info.vl_col * NBITS(panel_info.vl_bpix)) / 8;
	size = line_length * panel_info.vl_row;
	size += PAGE_SIZE;

	return size;
}

int rtsfb_enable_irq(void)
{
//      set_bit(LCDC_INT_EN, (1 << FRAME_START_INT_EN));
	set_bit(LCDC_INT_EN, (1 << FRAME_END_INT_EN));
//	set_bit(LCDC_INT_EN, (1 << SRAM_BUF_OVERFLOW_INT_EN));
#ifdef HSYNC_COUNT
	set_bit(LCDC_INT_EN, (1 << HSYNC_INT_EN));
#endif
	set_bit(LCDC_INT_EN, (1 << VSYNC_INT_EN));
	set_bit(LCDC_INT_EN, (1 << BUS_INT_EN));

	return 0;
}

static int rtsfb_init_mem(void *lcdbase, vidinfo_t *vid)
{
	u_long palette_mem_size;
	struct rtsfb_info *fbi = &vid->rts;
	int fb_size;
	int reg;

	/* open lcdc mem */
	reg = lcdc_read(SYS_MEM_LS);
	reg &= ~(1 << LCDC_MEM);
	lcdc_write(SYS_MEM_LS, reg);
	reg = lcdc_read(SYS_MEM_SD);
	reg &= ~(1 << LCDC_MEM);
	lcdc_write(SYS_MEM_SD, reg);

	fb_size = vid->vl_row * vid->vl_col * NBITS(vid->vl_bpix) / 8;
	fbi->screen = ((u_long)lcdbase);
	fbi->palette_size = 256 ;
	palette_mem_size = fbi->palette_size * sizeof(u32);

	debug("fb_size = 0x%p\n", (u_long) fb_size);
	/* locate palette and descs at end of page following fb */
	fbi->palette = (u_long)lcdbase + fb_size + PAGE_SIZE - palette_mem_size;
	debug("fbi palette %p\n", fbi->palette);

	return 0;
}

#ifdef CONFIG_RFH_495B

static int spi_gpio_init(void)
{
	u32 reg;

	reg = lcdc_read(GPIO_OE);
	reg |= 0x0b;
	lcdc_write(GPIO_OE, reg);

	return 0;
}

static int set_gpio(int port, ushort val)
{
	u32 reg;

	reg = lcdc_read(GPIO_VALUE);
	reg &= ~(1 << port);
	reg |= (val << port);
	lcdc_write(GPIO_VALUE, reg);
	return 0;
}

static void spi_send_data(unsigned char data)
{
	unsigned char n;

	for (n = 0; n < 8; n++) {
		if (data & 0x80)
			set_gpio(SPI_DI, 1);
		else
			set_gpio(SPI_DI, 0);
		data <<= 1;
		set_gpio(SPI_CLK, 0);
		set_gpio(SPI_CLK, 1);
	}
}

static void spi_write_cmd(unsigned char cmd)
{
	set_gpio(SPI_CS, 0);
	set_gpio(SPI_DI, 0);
	set_gpio(SPI_CLK, 0);
	set_gpio(SPI_CLK, 1);

	spi_send_data(cmd);

	set_gpio(SPI_CS, 1);
}

static void spi_write_data(unsigned char data)
{
	set_gpio(SPI_CS, 0);
	set_gpio(SPI_DI, 1);
	set_gpio(SPI_CLK, 0);
	set_gpio(SPI_CLK, 1);

	spi_send_data(data);

	set_gpio(SPI_CS, 1);
}

#endif

#ifdef CONFIG_CPU_MONAHANS
static inline void rtsfb_setup_gpio(vidinfo_t *vid) {}
#else
static void set_gpio_spi(void)
{
	spi_gpio_init();

	spi_write_cmd(0x11);
	mdelay(120);

	spi_write_cmd(0xff);
	spi_write_data(0x77);
	spi_write_data(0x01);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x13);

	spi_write_cmd(0xef);
	spi_write_data(0x08);

	spi_write_cmd(0xff);
	spi_write_data(0x77);
	spi_write_data(0x01);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x10);

	spi_write_cmd(0xc0);
	spi_write_data(0xe9);
	spi_write_data(0x03);

	spi_write_cmd(0xc1);
	spi_write_data(0x09);
	spi_write_data(0x02);

	spi_write_cmd(0xc2);
	spi_write_data(0x01);
	spi_write_data(0x08);

	spi_write_cmd(0xcc);
	spi_write_data(0x10);

	spi_write_cmd(0xb0);
	spi_write_data(0x00);
	spi_write_data(0x0b);
	spi_write_data(0x10);
	spi_write_data(0x0d);
	spi_write_data(0x11);
	spi_write_data(0x06);
	spi_write_data(0x01);
	spi_write_data(0x08);
	spi_write_data(0x08);
	spi_write_data(0x1d);
	spi_write_data(0x04);
	spi_write_data(0x10);
	spi_write_data(0x10);
	spi_write_data(0x27);
	spi_write_data(0x30);
	spi_write_data(0x19);

	spi_write_cmd(0xb1);
	spi_write_data(0x00);
	spi_write_data(0x0b);
	spi_write_data(0x14);
	spi_write_data(0x0c);
	spi_write_data(0x11);
	spi_write_data(0x05);
	spi_write_data(0x03);
	spi_write_data(0x08);
	spi_write_data(0x08);
	spi_write_data(0x20);
	spi_write_data(0x04);
	spi_write_data(0x13);
	spi_write_data(0x10);
	spi_write_data(0x28);
	spi_write_data(0x30);
	spi_write_data(0x19);

	spi_write_cmd(0xff);
	spi_write_data(0x77);
	spi_write_data(0x01);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x11);

	spi_write_cmd(0xb0);
	spi_write_data(0x35);

	spi_write_cmd(0xb1);
	spi_write_data(0x31);

	spi_write_cmd(0xb2);
	spi_write_data(0x82);

	spi_write_cmd(0xb3);
	spi_write_data(0x80);

	spi_write_cmd(0xb5);
	spi_write_data(0x4e);

	spi_write_cmd(0xb7);
	spi_write_data(0x85);

	spi_write_cmd(0xb8);
	spi_write_data(0x20);

	spi_write_cmd(0xb9);
	spi_write_data(0x10);

	spi_write_cmd(0xc1);
	spi_write_data(0x78);

	spi_write_cmd(0xc2);
	spi_write_data(0x78);

	spi_write_cmd(0xd0);
	spi_write_data(0x88);
	mdelay(100);

	spi_write_cmd(0xe0);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x02);

	spi_write_cmd(0xe1);
	spi_write_data(0x05);
	spi_write_data(0xa0);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x04);
	spi_write_data(0xa0);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x20);
	spi_write_data(0x20);

	spi_write_cmd(0xe2);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);

	spi_write_cmd(0xe3);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x33);
	spi_write_data(0x00);

	spi_write_cmd(0xe4);
	spi_write_data(0x22);
	spi_write_data(0x00);

	spi_write_cmd(0xe5);
	spi_write_data(0x08);
	spi_write_data(0x34);
	spi_write_data(0xa0);
	spi_write_data(0xa0);
	spi_write_data(0x06);
	spi_write_data(0x34);
	spi_write_data(0xa0);
	spi_write_data(0xa0);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);

	spi_write_cmd(0xe6);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x33);
	spi_write_data(0x00);

	spi_write_cmd(0xe7);
	spi_write_data(0x22);
	spi_write_data(0x00);

	spi_write_cmd(0xe8);
	spi_write_data(0x07);
	spi_write_data(0x34);
	spi_write_data(0xa0);
	spi_write_data(0xa0);
	spi_write_data(0x05);
	spi_write_data(0x34);
	spi_write_data(0xa0);
	spi_write_data(0xa0);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);

	spi_write_cmd(0xeb);
	spi_write_data(0x02);
	spi_write_data(0x00);
	spi_write_data(0x10);
	spi_write_data(0x10);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);

	spi_write_cmd(0xec);
	spi_write_data(0x02);
	spi_write_data(0x00);

	spi_write_cmd(0xed);
	spi_write_data(0xaa);
	spi_write_data(0x54);
	spi_write_data(0x0b);
	spi_write_data(0xbf);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xff);
	spi_write_data(0xfb);
	spi_write_data(0xb0);
	spi_write_data(0x45);
	spi_write_data(0xaa);

	spi_write_cmd(0xef);
	spi_write_data(0x10);
	spi_write_data(0x0d);
	spi_write_data(0x04);
	spi_write_data(0x08);
	spi_write_data(0x3f);
	spi_write_data(0x1f);

	spi_write_cmd(0xff);
	spi_write_data(0x77);
	spi_write_data(0x01);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_data(0x00);
	spi_write_cmd(0x29);
	mdelay(20);
}

static void rtsfb_setup_gpio(vidinfo_t *vid)
{
	/* enable */
	if (vid->mode == LVDS_INTERFACE) {
		set_bit(LCDC_FUNC, (1 << LVDS_MODE));
		lcdc_write(LVDS_PAD_CFG, 0x11111);
	} else if (vid->mode == RGB_INTERFACE) {
		lcdc_write(LVDS_PAD_CFG, 0x44444);
		lcdc_write(MIPI_TX_PAD_CFG, 0x88888);
	}
}
#endif

static void rtsfb_enable_controller(vidinfo_t *vid)
{
	struct rtsfb_info *fbi = &vid->rts;

	printf("Enabling LCD controller\n");

	/* reset */
	set_bit(LCDC_CTRL, (1 << LCDC_RESET));
	/* set panel size */
	lcdc_writereg_field(LCDC_PANEL, vid->vl_col, 16, 12);
	lcdc_writereg_field(LCDC_PANEL, vid->vl_row, 0, 12);

	set_bit(LCDC_CTRL, (1 << TRIG_LCDC_CTRL_START));
	printf("lcd start\n");
}

static void lcd_clk(vidinfo_t *vid)
{
	u32 divn;
	u32 divf;
	u32 rd_data;

	lcdc_write(LCDC_DRAM_CLK_CFG_R, CLK_ENABLE);
	lcdc_write(R_LVDS_ANA_TOP_CFG_8, 0);
	lcdc_write(R_POW_PIF_P1_TXL, 0);

	/*pxl_clk 30M -> lvds_pll 210M;
	 *pxl_clk 70M -> lvds_pll 490M;
	 *pxl_clk 10M -> lvds_pll 500M;
	 */
	if (vid->pxl_clk == 30000000) {
		divn = 6;
		divf = 1638;

	} else if (vid->pxl_clk == 70000000) {
		divn = 17;
		divf = 2458;
	} else if (vid->pxl_clk == 25000000) {
		divn = 5;
		divf = 0;
	} else {
		divn = 18;
		divf = 0;
	}
	lcdc_write(LVDS_SSCG_CFG1, (divn | (divf << 9)));
	lcdc_write(LVDS_SSCG_CFG2, 0x1);
	lcdc_write(R_LVDS_ANA_TOP_CFG_21, 0);
	rd_data = lcdc_read(R_LVDS_ANA_TOP_CFG_20);
	lcdc_write(R_LVDS_ANA_TOP_CFG_20, (0x3000000 + (rd_data & 0xffffff)));
	lcdc_write(R_LVDS_ANA_TOP_CFG_8, 0x1);
	lcdc_write(R_POW_PIF_P1_TXL, 0x3f);
	lcdc_write(R_LCDC_CLK_CFG_REG, CLK_ENABLE);
}

static int rtsfb_init(vidinfo_t *vid)
{
	u32 data;

	/* set LCD clock */
	lcd_clk(vid);
	/* set GPIO */
	rtsfb_setup_gpio(vid);

	printf("Configuring RTS LCD\n");

	/* setup info */
	lcdc_writereg_field(LCDC_FORMAT_CTRL, vid->vl_format, 8, 4);
	lcdc_write(LCDC_ALPHA, (255 << 16 | 255));
	lcdc_write(LCDC_BOTTOM_COLOR_CTRL, vid->bgcolor);
	lcdc_writereg_field(LCDC_FUNC, 1, 7, 1); //GAMMA FUNC
	if (vid->mode == RGB_INTERFACE) {
		/* DE & Vsync & Hsync output enable */
		lcdc_writereg_field(LCDC_FORMAT_CTRL, vid->rgb_outctrl, 12, 3);
	} else {
		lcdc_write(LCDC_LVDS_LANE_MAP, 0x43210); //lvds-signal-map
		lcdc_writereg_field(LCDC_FORMAT_CTRL, 0, 20, 5); //signal-swap
		lcdc_write(LCDC_LVDS_SIGNAL_SELECT, 0x423); //lvds-signal-select
		lcdc_writereg_field(LCDC_FORMAT_CTRL, vid->lvds_format, 0, 2); //lvds-format
	}
	/* set polarity */
	if (!vid->vl_hsp)
		lcdc_writereg_field(LCDC_FORMAT_CTRL, 1, 16, 1);
	if (!vid->vl_vsp)
		lcdc_writereg_field(LCDC_FORMAT_CTRL, 1, 17, 1);
	if (!vid->vl_dp)
		lcdc_writereg_field(LCDC_FORMAT_CTRL, 1, 18, 1);
	if (!vid->vl_clkp)
		lcdc_writereg_field(LCDC_FUNC, 1, 6, 1);

	/* set dummy */
	lcdc_write(LCDC_CFG_HSYNC_CYCLE, vid->hsw);
	lcdc_writereg_field(LCDC_DUMMY_START, vid->hbp + vid->hsw, 16, 12);
	lcdc_writereg_field(LCDC_DUMMY_END, vid->hfp, 16, 12);

	data = vid->vl_col + vid->hfp + vid->hbp + vid->hsw;
	data *= vid->vsw;
	lcdc_write(LCDC_CFG_VSYNC_CYCLE, data);
	lcdc_writereg_field(LCDC_DUMMY_START, vid->vbp + vid->vsw, 0, 12);
	lcdc_writereg_field(LCDC_DUMMY_END, vid->vfp, 0, 12);

	lcdc_write(LCDC_REG_LOAD, 1);

	return 0;
}

/************************************************************************/
#endif /* CONFIG_LCD */
