/*
 * rts_lcd.h - RTS LCD Controller structures
 *
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _RTS_LCD_H_
#define _RTS_LCD_H_

/*
 * RTS LCD info
 */
struct rtsfb_info {
	/* keep these registers in case we need to re-write palette */
	unsigned long screen;		/* physical address of frame buffer */
	unsigned long palette;
	unsigned int palette_size;
};

/*
 * RTS LCD controller structure
 */
typedef struct vidinfo {
	unsigned short	vl_col;		/* Number of columns (i.e. 640) */
	unsigned short	vl_row;		/* Number of rows (i.e. 480) */
	unsigned short  vl_rot;		/* Rotation of Display (0, 1, 2, 3) */
	unsigned int	pxl_clk;
	unsigned int	mode;		/*RGB or LVDS*/
	unsigned int	lvds_format;

	/* LCD configuration register */
	unsigned char	vl_clkp;	/* Clock polarity */
	unsigned char	vl_hsp;		/* Horizontal Sync polarity */
	unsigned char	vl_vsp;		/* Vertical Sync polarity */
	unsigned char	vl_dp;		/* Data polarity */
	unsigned char	rgb_outctrl;/* RGB output ctrl only for rgb interface */
	unsigned char	vl_bpix;	/* Bits per pixel*/
	unsigned char	vl_format;	/* Display format*/

	/* Horizontal control register. Timing from data sheet */
	unsigned short	hfp;		/* Horizontal front porch */
	unsigned short	hbp;		/* Horizontal back porch */
	unsigned short	hsw;		/* Horizontal Sync Pulse Width */

	/* Vertical control register. */
	unsigned short	vfp;		/* Vertical front porch */
	unsigned short	vbp;		/* Vertical back porch */
	unsigned short	vsw;		/* Vertical Sync Pulse Width */

	unsigned long	bgcolor;	/* bottom color */

	/* RTS LCD controller params */
	struct	rtsfb_info rts;
} vidinfo_t;

#endif  /* ifndef _RTS_LCD_H_ */

/*
 * Porting to u-boot:
 *
 * (C) Copyright 2011
 * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
 *
 * Copyright (C) 2008-2009 MontaVista Software Inc.
 * Copyright (C) 2008-2009 Texas Instruments Inc
 *
 * Based on the LCD driver for TI Avalanche processors written by
 * Ajay Singh and Shalom Hai.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef RTSFB_H
#define RTSFB_H

#define RTS_LCD_CNTL_BASE	0xb8c00000
#define SYSLCDC_REG(x)	((x) + RTS_LCD_CNTL_BASE)
#define LCDC_CTRL			SYSLCDC_REG(0x0)
#define LCDC_FUNC			 SYSLCDC_REG(0x4)
#define LCDC_FORMAT_CTRL                 SYSLCDC_REG(0x8)
#define LCDC_HWM                         SYSLCDC_REG(0xC)
#define LCDC_SRAM_ACCESS_LEN             SYSLCDC_REG(0x10)
#define LCDC_PANEL                       SYSLCDC_REG(0x14)
#define LCDC_DUMMY_START                 SYSLCDC_REG(0x18)
#define LCDC_DUMMY_END                   SYSLCDC_REG(0x1C)
#define LCDC_CFG_VSYNC_CYCLE             SYSLCDC_REG(0x20)
#define LCDC_CFG_HSYNC_CYCLE             SYSLCDC_REG(0x24)
#define LCDC_DISPLAY_VIDEO_OFFSET        SYSLCDC_REG(0x28)
#define LCDC_DDR_VIDEO_OFFSET            SYSLCDC_REG(0x2C)
#define LCDC_DISPLAY_VIDEO_SIZE          SYSLCDC_REG(0x30)
#define LCDC_DDR_VIDEO_SIZE              SYSLCDC_REG(0x34)
#define LCDC_VIDEO_ADDR_DDR              SYSLCDC_REG(0x38)
#define LCDC_VIDEO_ADDR_DDR_UV           SYSLCDC_REG(0x3C)
#define LCDC_VIDEO_ADDR_DDR_SET          SYSLCDC_REG(0x40)
#define LCDC_VIDEO_ADDR_DDR_SET_UV       SYSLCDC_REG(0x44)
#define LCDC_VIDEO_ADDR_DDR_SET_VLD      SYSLCDC_REG(0x48)
#define LCDC_DISPLAY_GRAPHIC_OFFSET      SYSLCDC_REG(0x4C)
#define LCDC_DDR_GRAPHIC_OFFSET          SYSLCDC_REG(0x50)
#define LCDC_DISPLAY_GRAPHIC_SIZE        SYSLCDC_REG(0x54)
#define LCDC_DDR_GRAPHIC_SIZE            SYSLCDC_REG(0x58)
#define LCDC_GRAPHIC_ADDR_DDR            SYSLCDC_REG(0x5C)
#define LCDC_GRAPHIC_ADDR_DDR_UV         SYSLCDC_REG(0x60)
#define LCDC_GRAPHIC_ADDR_DDR_SET        SYSLCDC_REG(0x64)
#define LCDC_GRAPHIC_ADDR_DDR_SET_UV     SYSLCDC_REG(0x68)
#define LCDC_GRAPHIC_ADDR_DDR_SET_VLD    SYSLCDC_REG(0x6C)
#define LCDC_ALPHA                       SYSLCDC_REG(0x70)
#define LCDC_BOTTOM_COLOR_CTRL           SYSLCDC_REG(0x74)
#define LCDC_LVDS_SIGNAL_SELECT          SYSLCDC_REG(0x78)
#define LCDC_LVDS_LANE_MAP               SYSLCDC_REG(0x7C)
#define LCDC_GAMMA_P_0                   SYSLCDC_REG(0x80)
#define LCDC_GAMMA_P_1                   SYSLCDC_REG(0x84)
#define LCDC_GAMMA_P_2                   SYSLCDC_REG(0x88)
#define LCDC_GAMMA_P_3                   SYSLCDC_REG(0x8C)
#define LCDC_GAMMA_P_4                   SYSLCDC_REG(0x90)
#define LCDC_GAMMA_P_5                   SYSLCDC_REG(0x94)
#define LCDC_GAMMA_P_6                   SYSLCDC_REG(0x98)
#define LCDC_GRAPHIC_COLOR_KEY_MAX      SYSLCDC_REG(0x9C)
#define LCDC_GRAPHIC_COLOR_KEY_MIN      SYSLCDC_REG(0xA0)
#define LCDC_GRAPHIC_COLOR_KEY_MASK     SYSLCDC_REG(0xA4)
#define LCDC_VIDEO_COLOR_KEY_MAX        SYSLCDC_REG(0xA8)
#define LCDC_VIDEO_COLOR_KEY_MIN        SYSLCDC_REG(0xAC)
#define LCDC_VIDEO_COLOR_KEY_MASK       SYSLCDC_REG(0xB0)
#define LCDC_REG_LOAD                   SYSLCDC_REG(0xB4)
#define LCDC_INT_EN			 SYSLCDC_REG(0xB8)
#define LCDC_INT_FLAG			 SYSLCDC_REG(0xBC)
#define R_CURSOR_OFFSET					SYSLCDC_REG(0xC0)
#define R_CURSOR_RGB0_CFG				SYSLCDC_REG(0xC4)
#define R_CURSOR_RGB1_CFG				SYSLCDC_REG(0xC8)
#define R_CURSOR_RGB2_CFG				SYSLCDC_REG(0xCC)
#define R_CURSOR_LOOKUP_TABLE_SEL		SYSLCDC_REG(0xD0)
#define R_LVDS_TEST_PATTERN				SYSLCDC_REG(0xD4)
#define LCDC_ARGB_LOOKUP_TABLE            SYSLCDC_REG(0x200)

#define	LCDC_RESET					(2)
#define	TRIG_LCDC_CTRL_STOP				(1)
#define	TRIG_LCDC_CTRL_START				(0)

#define ENABLE_CURSOR_EN					(14)
#define	GRAPHIC_COLORKEY_MODE				(13)
#define	GRAPHIC_COLORKEY_EN				(12)
#define	VIDEO_COLORKEY_MODE				(11)
#define	VIDEO_COLORKEY_EN					(10)
#define	ENABLE_LFSR_MODE					(9)
#define	ENABLE_DITHERING					(8)
#define	ENABLE_GAMMA_FUNCTION				(7)
#define	PIXEL_CLOCK_OUT_POLARITY_NEG		(6)
#define	LVDS_MODE							(5)
#define	GRAPHIC_DISPLAY_ENABLE_SIGNAL	(4)
#define	VIDEO_DISPLAY_ENABLE_SIGNAL		(3)
#define	YUV444_TO_RGB888_YMODE_MODE		(2)
#define	TX_STOP_MODE						(1)
#define	YUV444_TO_RGB888_YBT709_MODE		(0)

#define FORMAT_PSEUDOCOLOR	(0)
#define FORMAT_RGB_565		(1)
#define FORMAT_ARGB_1555	(2)
#define FORMAT_ARGB_4444	(3)
#define FORMAT_ARGB_8888	(4)
#define FORMAT_BGRA_8888	(5)
#define FORMAT_YUV420_PACKED (9)
#define RGB888		(0)
#define RGB666		(1)
#define RGB565		(2)
#define GRAPHIC_DISPLAY_ENABLE_SIGNAL	(4)

#define LVDS_8BIT_VESA	(0)
#define LVDS_8BIT_JEIDA	(1)
#define LVDS_6BIT_VESA	(2)
#define LVDS_6BIT_JEIDA	(3)

#define FRAME_START_INT_EN			(5)
#define FRAME_END_INT_EN			(4)
#define SRAM_BUF_OVERFLOW_INT_EN		(3)
#define HSYNC_INT_EN				(2)
#define VSYNC_INT_EN				(1)
#define BUS_INT_EN					(0)

#define FRAME_START_INT_FLAG		(5)
#define FRAME_END_INT_FLAG			(4)
#define SRAM_BUF_OVERFLOW_INT_FLAG	(3)
#define HSYNC_INT_FLAG					(2)
#define VSYNC_INT_FLAG				(1)
#define BUS_INT_FLAG					(0)

#define CLOCK_LANE_SWAP				(24)
#define LOGIC_DATA_LANE3_SWAP			(23)
#define LOGIC_DATA_LANE2_SWAP			(22)
#define LOGIC_DATA_LANE1_SWAP			(21)
#define LOGIC_DATA_LANE0_SWAP			(20)
#define DE_POLARITY_NEG			(18)
#define VSYNC_POLARITY_NEG		(17)
#define HSYNC_POLARITY_NEG		(16)
#define DE_OUTPUT_ENABLE			(14)
#define VSYNC_OUTPUT_ENABLE		(13)
#define HSYNC_OUTPUT_ENABLE		(12)

/* for LCD Panel RFH495BFW
 * GPIO0---SDI
 * GPIO1---SCL
 * GPIO3---CS
 */
#define GPIO_BASE				0xb8800000
#define SYSGPIO_REG(x)			((x) + GPIO_BASE)
#define GPIO_OE					SYSGPIO_REG(0x0000)
#define GPIO_VALUE				SYSGPIO_REG(0x0004)
#define MIPI_TX_PAD_CFG			SYSGPIO_REG(0x01cc)
#define LVDS_PAD_CFG			SYSGPIO_REG(0x01ec)

#define SPI_DI						0
#define SPI_CLK						1
#define SPI_CS						3

#define LVDS_INTERFACE 1
#define RGB_INTERFACE 0

#define LCDC_DRAM_CLK_CFG_R			0xb88d00c0

#define LVDS_BASE_RF				0xb8c80000
#define SYSLVDSPLL_REG(x)			((x) + LVDS_BASE_RF)
#define R_LVDS_ANA_TOP_CFG_0		SYSLVDSPLL_REG(0x00)
#define R_LVDS_ANA_TOP_CFG_1		SYSLVDSPLL_REG(0x04)
#define R_LVDS_ANA_TOP_CFG_2		SYSLVDSPLL_REG(0x08)
#define R_LVDS_ANA_TOP_CFG_3		SYSLVDSPLL_REG(0x0C)
#define R_LVDS_ANA_TOP_CFG_4		SYSLVDSPLL_REG(0x10)
#define R_LVDS_ANA_TOP_CFG_5		SYSLVDSPLL_REG(0x14)
#define R_LVDS_ANA_TOP_CFG_6		SYSLVDSPLL_REG(0x18)
#define R_LVDS_ANA_TOP_CFG_7		SYSLVDSPLL_REG(0x1C)
#define R_LVDS_ANA_TOP_CFG_8		SYSLVDSPLL_REG(0x20)
#define R_LVDS_ANA_TOP_CFG_9		SYSLVDSPLL_REG(0x24)
#define R_LVDS_ANA_TOP_CFG_10		SYSLVDSPLL_REG(0x28)
#define R_LVDS_ANA_TOP_CFG_11		SYSLVDSPLL_REG(0x2C)
#define R_LVDS_ANA_TOP_CFG_12		SYSLVDSPLL_REG(0x30)
#define R_LVDS_ANA_TOP_CFG_13		SYSLVDSPLL_REG(0x34)
#define R_LVDS_ANA_TOP_CFG_14		SYSLVDSPLL_REG(0x38)
#define R_LVDS_ANA_TOP_CFG_15		SYSLVDSPLL_REG(0x3C)
#define R_LVDS_ANA_TOP_CFG_16		SYSLVDSPLL_REG(0x40)
#define R_LVDS_ANA_TOP_CFG_17		SYSLVDSPLL_REG(0x44)
#define R_LVDS_ANA_TOP_CFG_18		SYSLVDSPLL_REG(0x48)
#define R_LVDS_ANA_TOP_CFG_19		SYSLVDSPLL_REG(0x4C)
#define R_LVDS_ANA_TOP_CFG_20		SYSLVDSPLL_REG(0x50)
#define R_LVDS_ANA_TOP_CFG_21		SYSLVDSPLL_REG(0x54)
#define R_LVDS_ANA_TOP_CFG_22		SYSLVDSPLL_REG(0x58)
#define R_POW_PIF_P1_TXL			SYSLVDSPLL_REG(0x5c)
#define R_LVDS_ANA_GPIO_SEL			SYSLVDSPLL_REG(0x60)
#define R_LVDS_ANA_GPIO_O			SYSLVDSPLL_REG(0x64)
#define R_LCDC_CLK_CFG_REG			SYSLVDSPLL_REG(0x68)
#define R_FPGA_CLK_CFG_REG			SYSLVDSPLL_REG(0x6C)
#define LVDS_SSCG_CFG0				SYSLVDSPLL_REG(0x70)
#define LVDS_SSCG_CFG1				SYSLVDSPLL_REG(0x74)
#define LVDS_SSCG_CFG2				SYSLVDSPLL_REG(0x78)

#define SYS_MEM_LS					0xB88C0220
#define SYS_MEM_SD					0xB88C0228
#define LCDC_MEM					(19)

#define CLK_ENABLE		0x1000000

#endif  /* ifndef RTSFB_H */
