/*
 * (C) Copyright 2012 SAMSUNG Electronics
 * Jaehoon Chung <jh80.chung@samsung.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef __DWMMC_HW_H
#define __DWMMC_HW_H

#include <asm/io.h>
#include <mmc.h>

#define writeb(b, addr) (*(volatile unsigned char *)(addr)) = (b)
#define writew(b, addr) (*(volatile unsigned short *)(addr)) = (b)
#define writel(b, addr) (*(volatile unsigned int *)(addr)) = (b)

#define readb(addr) (*(volatile unsigned char *)(addr))
#define readw(addr) (*(volatile unsigned short *)(addr))
#define readl(addr) (*(volatile unsigned int *)(addr))

#define RTSX_SD_DEFAULT_MODE		0
#define RTSX_SD_HS_MODE			1
#define RTSX_SD_SDR_MODE		2
#define RTSX_SD_DDR_MODE		3
#define RTSX_ICR_STATE_IDLE		0
#define RTSX_ICR_STATE_RUN		1
#define RTSX_SD_POWER_OFF		0
#define RTSX_SD_POWER_ON		1

#ifdef CONFIG_BOARD_RTS3915
#ifdef CONFIG_RTS3915_BGA325_DDR3_GENERAL
#define RTS_MMC_GPIO_DIR 0xb88000c0
#define RTS_MMC_GPIO_OUT 0xb88000c4
#else
#define RTS_MMC_GPIO_DIR 0xb8800000
#define RTS_MMC_GPIO_OUT 0xb8800004
#endif
#define RTS_MMC_BASEADDR 0xb8300000
#else
#define RTS_MMC_GPIO_DIR 0xb8800000
#define RTS_MMC_GPIO_OUT 0xb8800008
#define RTS_MMC_BASEADDR 0xb8300000
#endif

#define MMC_TIMING_LEGACY	0
#define MMC_TIMING_MMC_HS	1
#define MMC_TIMING_SD_HS	2
#define MMC_TIMING_UHS_SDR12	3
#define MMC_TIMING_UHS_SDR25	4
#define MMC_TIMING_UHS_SDR50	5
#define MMC_TIMING_UHS_SDR104	6
#define MMC_TIMING_UHS_DDR50	7
#define MMC_TIMING_MMC_HS200	8

#define MMC_BUS_WIDTH_1	1
#define MMC_BUS_WIDTH_4	4
#define MMC_BUS_WIDTH_8	8

#define CMD_TYPE_CMD_BUFF	0x1
#define CMD_TYPE_DATA_READ	0x2
#define CMD_TYPE_DATA_WRITE	0x4
#define CMD_TYPE_DATA_SREAD	0x8
#define CMD_TYPE_DATA_SWRITE	0x10

struct rtsmmc_host {
	struct mmc_config	cfg;
	int			id;
	uint32_t		buswidth;

	void			*resv_buf;

	u32			cmd_idx;
	dma_addr_t		cmd_addr;
	void			*cmd_ptr;

	u32			sg_tbl_idx;
	dma_addr_t		sg_tbl_addr;
	void			*sg_tbl_ptr;

	u32			bier;
	u32			card_status;

	bool			removed;
	bool			short_dma_mode;

	const u32		*sd_pull_ctl_enable_tbl;
	const u32		*sd_pull_ctl_disable_tbl;

	u8			state;

	u32			cmd_timeout_ms;
	u32			data_timeout_ms;

	struct mmc		*mmc;

	bool			initial_mode;
	unsigned int		clock;
	unsigned int		max_clock;
	unsigned int		min_clock;
	u8			sd_mode;

	u32			data_errors;
};

static inline void rtsmmc_writel(int reg, u32 val)
{
	writel(val, RTS_MMC_BASEADDR + reg);
}

static inline void rtsmmc_writew(int reg, u16 val)
{
	writew(val, RTS_MMC_BASEADDR + reg);
}

static inline void rtsmmc_writeb(int reg, u8 val)
{
	writeb(val, RTS_MMC_BASEADDR + reg);
}
static inline u32 rtsmmc_readl(int reg)
{
	return readl(RTS_MMC_BASEADDR + reg);
}

static inline u16 rtsmmc_readw(int reg)
{
	return readw(RTS_MMC_BASEADDR + reg);
}

static inline u8 rtsmmc_readb(int reg)
{
	return readb(RTS_MMC_BASEADDR + reg);
}

int add_rtsmmc(struct rtsmmc_host *host, u32 max_clk, u32 min_clk);
#endif	/* __DWMMC_HW_H */
